@@ -44,13 +44,16 @@ import org.apache.celeborn.common.network.client.RpcResponseCallback;
 import org.apache.celeborn.common.network.client.TransportClient;
 import org.apache.celeborn.common.network.client.TransportClientFactory;
 import org.apache.celeborn.common.network.protocol.PushData;
-import org.apache.celeborn.common.network.protocol.PushDataHandShake;
-import org.apache.celeborn.common.network.protocol.RegionFinish;
-import org.apache.celeborn.common.network.protocol.RegionStart;
+import org.apache.celeborn.common.network.protocol.TransportMessage;
 import org.apache.celeborn.common.network.util.TransportConf;
+import org.apache.celeborn.common.protocol.MessageType;
 import org.apache.celeborn.common.protocol.PartitionLocation;
 import org.apache.celeborn.common.protocol.PbChangeLocationPartitionInfo;
 import org.apache.celeborn.common.protocol.PbChangeLocationResponse;
+import org.apache.celeborn.common.protocol.PbPartitionLocation.Mode;
+import org.apache.celeborn.common.protocol.PbPushDataHandShake;
+import org.apache.celeborn.common.protocol.PbRegionFinish;
+import org.apache.celeborn.common.protocol.PbRegionStart;
 import org.apache.celeborn.common.protocol.ReviveRequest;
 import org.apache.celeborn.common.protocol.TransportModuleConstants;
 import org.apache.celeborn.common.protocol.message.ControlMessages;
@@ -125,7 +128,7 @@ public class FlinkShuffleClientImpl extends ShuffleClientImpl {
       long driverTimestamp,
       CelebornConf conf,
       UserIdentifier userIdentifier) {
-    super(appUniqueId, conf, userIdentifier, false);
+    super(appUniqueId, conf, userIdentifier);
     String module = TransportModuleConstants.DATA_MODULE;
     TransportConf dataTransportConf =
         Utils.fromCelebornConf(conf, module, conf.getInt("celeborn." + module + ".io.threads", 8));
@@ -148,12 +151,19 @@ public class FlinkShuffleClientImpl extends ShuffleClientImpl {
       logger.error("Shuffle data is empty for shuffle {} partitionId {}.", shuffleId, partitionId);
       throw new PartitionUnRetryAbleException(partitionId + " may be lost.");
     } else {
+      PartitionLocation[] partitionLocations =
+          fileGroups.partitionGroups.get(partitionId).toArray(new PartitionLocation[0]);
+      Arrays.sort(partitionLocations, Comparator.comparingInt(PartitionLocation::getEpoch));
+      logger.debug(
+          "readBufferedPartition shuffleKey:{} partitionid:{} partitionLocation:{}",
+          shuffleKey,
+          partitionId,
+          partitionLocations);
       return CelebornBufferStream.create(
           this,
-          conf,
           flinkTransportClientFactory,
           shuffleKey,
-          fileGroups.partitionGroups.get(partitionId).toArray(new PartitionLocation[0]),
+          partitionLocations,
           subPartitionIndexStart,
           subPartitionIndexEnd);
     }
@@ -177,7 +187,11 @@ public class FlinkShuffleClientImpl extends ShuffleClientImpl {
         } else {
           // refresh file groups
           ReduceFileGroups newGroups = loadFileGroupInternal(shuffleId);
-          if (newGroups == null || !newGroups.partitionIds.contains(partitionId)) {
+          if (newGroups == null) {
+            throw new IOException(
+                "Load file group from lifecycle manager failed: "
+                    + Utils.makeReducerKey(shuffleId, partitionId));
+          } else if (!newGroups.partitionIds.contains(partitionId)) {
             throw new IOException(
                 "shuffle data lost for partition: " + Utils.makeReducerKey(shuffleId, partitionId));
           }
@@ -301,7 +315,7 @@ public class FlinkShuffleClientImpl extends ShuffleClientImpl {
     return currentClient.get(mapKey);
   }
 
-  public void pushDataHandShake(
+  public Optional<PartitionLocation> pushDataHandShake(
       int shuffleId,
       int mapId,
       int attemptId,
@@ -311,12 +325,7 @@ public class FlinkShuffleClientImpl extends ShuffleClientImpl {
       throws IOException {
     final String mapKey = Utils.makeMapKey(shuffleId, mapId, attemptId);
     final PushState pushState = pushStates.computeIfAbsent(mapKey, (s) -> new PushState(conf));
-    sendMessageInternal(
-        shuffleId,
-        mapId,
-        attemptId,
-        location,
-        pushState,
+    return retrySendMessage(
         () -> {
           String shuffleKey = Utils.makeShuffleKey(appUniqueId, shuffleId);
           logger.info(
@@ -326,16 +335,33 @@ public class FlinkShuffleClientImpl extends ShuffleClientImpl {
               location.getUniqueId());
           logger.debug("PushDataHandShake location {}", location);
           TransportClient client = createClientWaitingInFlightRequest(location, mapKey, pushState);
-          PushDataHandShake handShake =
-              new PushDataHandShake(
-                  PRIMARY_MODE,
-                  shuffleKey,
-                  location.getUniqueId(),
-                  attemptId,
-                  numPartitions,
-                  bufferSize);
-          client.sendRpcSync(handShake.toByteBuffer(), conf.pushDataTimeoutMs());
-          return null;
+          ByteBuffer pushDataHandShakeResponse;
+          try {
+            pushDataHandShakeResponse =
+                client.sendRpcSync(
+                    new TransportMessage(
+                            MessageType.PUSH_DATA_HAND_SHAKE,
+                            PbPushDataHandShake.newBuilder()
+                                .setMode(Mode.forNumber(PRIMARY_MODE))
+                                .setShuffleKey(shuffleKey)
+                                .setPartitionUniqueId(location.getUniqueId())
+                                .setAttemptId(attemptId)
+                                .setNumPartitions(numPartitions)
+                                .setBufferSize(bufferSize)
+                                .build()
+                                .toByteArray())
+                        .toByteBuffer(),
+                    conf.pushDataTimeoutMs());
+          } catch (IOException e) {
+            // ioexeption revive
+            return revive(shuffleId, mapId, attemptId, location);
+          }
+          if (pushDataHandShakeResponse.hasRemaining()
+              && pushDataHandShakeResponse.get() == StatusCode.HARD_SPLIT.getValue()) {
+            // if split then revive
+            return revive(shuffleId, mapId, attemptId, location);
+          }
+          return Optional.empty();
         });
   }
 
@@ -349,12 +375,7 @@ public class FlinkShuffleClientImpl extends ShuffleClientImpl {
       throws IOException {
     final String mapKey = Utils.makeMapKey(shuffleId, mapId, attemptId);
     final PushState pushState = pushStates.computeIfAbsent(mapKey, (s) -> new PushState(conf));
-    return sendMessageInternal(
-        shuffleId,
-        mapId,
-        attemptId,
-        location,
-        pushState,
+    return retrySendMessage(
         () -> {
           String shuffleKey = Utils.makeShuffleKey(appUniqueId, shuffleId);
           logger.info(
@@ -365,69 +386,82 @@ public class FlinkShuffleClientImpl extends ShuffleClientImpl {
               location.getUniqueId());
           logger.debug("RegionStart  for location {}.", location.toString());
           TransportClient client = createClientWaitingInFlightRequest(location, mapKey, pushState);
-          RegionStart regionStart =
-              new RegionStart(
-                  PRIMARY_MODE,
-                  shuffleKey,
-                  location.getUniqueId(),
-                  attemptId,
-                  currentRegionIdx,
-                  isBroadcast);
-          ByteBuffer regionStartResponse =
-              client.sendRpcSync(regionStart.toByteBuffer(), conf.pushDataTimeoutMs());
+          ByteBuffer regionStartResponse;
+          try {
+            regionStartResponse =
+                client.sendRpcSync(
+                    new TransportMessage(
+                            MessageType.REGION_START,
+                            PbRegionStart.newBuilder()
+                                .setMode(Mode.forNumber(PRIMARY_MODE))
+                                .setShuffleKey(shuffleKey)
+                                .setPartitionUniqueId(location.getUniqueId())
+                                .setAttemptId(attemptId)
+                                .setCurrentRegionIndex(currentRegionIdx)
+                                .setIsBroadcast(isBroadcast)
+                                .build()
+                                .toByteArray())
+                        .toByteBuffer(),
+                    conf.pushDataTimeoutMs());
+          } catch (IOException e) {
+            // ioexeption revive
+            return revive(shuffleId, mapId, attemptId, location);
+          }
+
           if (regionStartResponse.hasRemaining()
               && regionStartResponse.get() == StatusCode.HARD_SPLIT.getValue()) {
             // if split then revive
-            Set<Integer> mapIds = new HashSet<>();
-            mapIds.add(mapId);
-            List<ReviveRequest> requests = new ArrayList<>();
-            ReviveRequest req =
-                new ReviveRequest(
-                    shuffleId,
-                    mapId,
-                    attemptId,
-                    location.getId(),
-                    location.getEpoch(),
-                    location,
-                    StatusCode.HARD_SPLIT);
-            requests.add(req);
-            PbChangeLocationResponse response =
-                lifecycleManagerRef.askSync(
-                    ControlMessages.Revive$.MODULE$.apply(shuffleId, mapIds, requests),
-                    conf.clientRpcRequestPartitionLocationRpcAskTimeout(),
-                    ClassTag$.MODULE$.apply(PbChangeLocationResponse.class));
-            // per partitionKey only serve single PartitionLocation in Client Cache.
-            PbChangeLocationPartitionInfo partitionInfo = response.getPartitionInfo(0);
-            StatusCode respStatus = Utils.toStatusCode(partitionInfo.getStatus());
-            if (StatusCode.SUCCESS.equals(respStatus)) {
-              return Optional.of(
-                  PbSerDeUtils.fromPbPartitionLocation(partitionInfo.getPartition()));
-            } else {
-              // throw exception
-              logger.error(
-                  "Exception raised while reviving for shuffle {} map {} attemptId {} partition {} epoch {}.",
-                  shuffleId,
-                  mapId,
-                  attemptId,
-                  location.getId(),
-                  location.getEpoch());
-              throw new CelebornIOException("RegionStart revive failed");
-            }
+            return revive(shuffleId, mapId, attemptId, location);
           }
           return Optional.empty();
         });
   }
 
+  public Optional<PartitionLocation> revive(
+      int shuffleId, int mapId, int attemptId, PartitionLocation location)
+      throws CelebornIOException {
+    Set<Integer> mapIds = new HashSet<>();
+    mapIds.add(mapId);
+    List<ReviveRequest> requests = new ArrayList<>();
+    ReviveRequest req =
+        new ReviveRequest(
+            shuffleId,
+            mapId,
+            attemptId,
+            location.getId(),
+            location.getEpoch(),
+            location,
+            StatusCode.HARD_SPLIT);
+    requests.add(req);
+    PbChangeLocationResponse response =
+        lifecycleManagerRef.askSync(
+            ControlMessages.Revive$.MODULE$.apply(shuffleId, mapIds, requests),
+            conf.clientRpcRequestPartitionLocationRpcAskTimeout(),
+            ClassTag$.MODULE$.apply(PbChangeLocationResponse.class));
+    // per partitionKey only serve single PartitionLocation in Client Cache.
+    PbChangeLocationPartitionInfo partitionInfo = response.getPartitionInfo(0);
+    StatusCode respStatus = Utils.toStatusCode(partitionInfo.getStatus());
+    if (StatusCode.SUCCESS.equals(respStatus)) {
+      logger.debug("revive new partition:{}", partitionInfo.getPartition());
+      return Optional.of(PbSerDeUtils.fromPbPartitionLocation(partitionInfo.getPartition()));
+    } else {
+      // throw exception
+      logger.error(
+          "Exception raised while reviving for shuffle {} map {} attemptId {} partition {} epoch {}.",
+          shuffleId,
+          mapId,
+          attemptId,
+          location.getId(),
+          location.getEpoch());
+      throw new CelebornIOException("RegionStart revive failed");
+    }
+  }
+
   public void regionFinish(int shuffleId, int mapId, int attemptId, PartitionLocation location)
       throws IOException {
     final String mapKey = Utils.makeMapKey(shuffleId, mapId, attemptId);
     final PushState pushState = pushStates.computeIfAbsent(mapKey, (s) -> new PushState(conf));
-    sendMessageInternal(
-        shuffleId,
-        mapId,
-        attemptId,
-        location,
-        pushState,
+    retrySendMessage(
         () -> {
           final String shuffleKey = Utils.makeShuffleKey(appUniqueId, shuffleId);
           logger.info(
@@ -438,38 +472,22 @@ public class FlinkShuffleClientImpl extends ShuffleClientImpl {
               location.getUniqueId());
           logger.debug("RegionFinish for location {}.", location);
           TransportClient client = createClientWaitingInFlightRequest(location, mapKey, pushState);
-          RegionFinish regionFinish =
-              new RegionFinish(PRIMARY_MODE, shuffleKey, location.getUniqueId(), attemptId);
-          client.sendRpcSync(regionFinish.toByteBuffer(), conf.pushDataTimeoutMs());
+          client.sendRpcSync(
+              new TransportMessage(
+                      MessageType.REGION_FINISH,
+                      PbRegionFinish.newBuilder()
+                          .setMode(Mode.forNumber(PRIMARY_MODE))
+                          .setShuffleKey(shuffleKey)
+                          .setPartitionUniqueId(location.getUniqueId())
+                          .setAttemptId(attemptId)
+                          .build()
+                          .toByteArray())
+                  .toByteBuffer(),
+              conf.pushDataTimeoutMs());
           return null;
         });
   }
 
-  private <R> R sendMessageInternal(
-      int shuffleId,
-      int mapId,
-      int attemptId,
-      PartitionLocation location,
-      PushState pushState,
-      ThrowingExceptionSupplier<R, Exception> supplier)
-      throws IOException {
-    int batchId = 0;
-    try {
-      // mapKey
-      final String mapKey = Utils.makeMapKey(shuffleId, mapId, attemptId);
-      pushState = getPushState(mapKey);
-
-      // add inFlight requests
-      batchId = pushState.nextBatchId();
-      pushState.addBatch(batchId, location.hostAndPushPort());
-      return retrySendMessage(supplier);
-    } finally {
-      if (pushState != null) {
-        pushState.removeBatch(batchId, location.hostAndPushPort());
-      }
-    }
-  }
-
   @FunctionalInterface
   interface ThrowingExceptionSupplier<R, E extends Exception> {
     R get() throws E;
