@@ -31,11 +31,18 @@ import org.apache.flink.configuration.Configuration;
 import org.apache.flink.configuration.MemorySize;
 import org.apache.flink.runtime.clusterframework.types.ResourceID;
 import org.apache.flink.runtime.executiongraph.ExecutionAttemptID;
+import org.apache.flink.runtime.executiongraph.ExecutionGraphID;
 import org.apache.flink.runtime.io.network.partition.ResultPartitionID;
 import org.apache.flink.runtime.io.network.partition.ResultPartitionType;
 import org.apache.flink.runtime.jobgraph.IntermediateDataSetID;
 import org.apache.flink.runtime.jobgraph.IntermediateResultPartitionID;
-import org.apache.flink.runtime.shuffle.*;
+import org.apache.flink.runtime.jobgraph.JobVertexID;
+import org.apache.flink.runtime.scheduler.strategy.ExecutionVertexID;
+import org.apache.flink.runtime.shuffle.JobShuffleContext;
+import org.apache.flink.runtime.shuffle.PartitionDescriptor;
+import org.apache.flink.runtime.shuffle.ProducerDescriptor;
+import org.apache.flink.runtime.shuffle.ShuffleMasterContext;
+import org.apache.flink.runtime.shuffle.TaskInputsOutputsDescriptor;
 import org.junit.After;
 import org.junit.Assert;
 import org.junit.Before;
@@ -166,6 +173,8 @@ public class RemoteShuffleMasterTest {
   public void testShuffleMemoryAnnouncing() {
     Map<IntermediateDataSetID, Integer> numberOfInputGateChannels = new HashMap<>();
     Map<IntermediateDataSetID, Integer> numbersOfResultSubpartitions = new HashMap<>();
+    Map<IntermediateDataSetID, Integer> subPartitionNums = new HashMap<>();
+    Map<IntermediateDataSetID, ResultPartitionType> inputPartitionTypes = new HashMap<>();
     Map<IntermediateDataSetID, ResultPartitionType> resultPartitionTypes = new HashMap<>();
     IntermediateDataSetID inputDataSetID0 = new IntermediateDataSetID();
     IntermediateDataSetID inputDataSetID1 = new IntermediateDataSetID();
@@ -175,23 +184,36 @@ public class RemoteShuffleMasterTest {
     Random random = new Random();
     numberOfInputGateChannels.put(inputDataSetID0, random.nextInt(1000));
     numberOfInputGateChannels.put(inputDataSetID1, random.nextInt(1000));
-    numbersOfResultSubpartitions.put(outputDataSetID0, random.nextInt(1000));
-    numbersOfResultSubpartitions.put(outputDataSetID1, random.nextInt(1000));
-    numbersOfResultSubpartitions.put(outputDataSetID2, random.nextInt(1000));
+    int subPartitionNum1 = random.nextInt(1000);
+    int subPartitionNum2 = random.nextInt(1000);
+    int subPartitionNum3 = random.nextInt(1000);
+    numbersOfResultSubpartitions.put(outputDataSetID0, subPartitionNum1);
+    numbersOfResultSubpartitions.put(outputDataSetID1, subPartitionNum2);
+    numbersOfResultSubpartitions.put(outputDataSetID2, subPartitionNum3);
+    subPartitionNums.put(outputDataSetID0, subPartitionNum1);
+    subPartitionNums.put(outputDataSetID1, subPartitionNum2);
+    subPartitionNums.put(outputDataSetID2, subPartitionNum3);
+    inputPartitionTypes.put(inputDataSetID0, ResultPartitionType.BLOCKING);
+    inputPartitionTypes.put(inputDataSetID1, ResultPartitionType.BLOCKING);
     resultPartitionTypes.put(outputDataSetID0, ResultPartitionType.BLOCKING);
     resultPartitionTypes.put(outputDataSetID1, ResultPartitionType.BLOCKING);
     resultPartitionTypes.put(outputDataSetID2, ResultPartitionType.BLOCKING);
     MemorySize calculated =
         remoteShuffleMaster.computeShuffleMemorySizeForTask(
             TaskInputsOutputsDescriptor.from(
-                numberOfInputGateChannels, numbersOfResultSubpartitions, resultPartitionTypes));
+                3,
+                numberOfInputGateChannels,
+                numbersOfResultSubpartitions,
+                subPartitionNums,
+                inputPartitionTypes,
+                resultPartitionTypes));
 
-    CelebornConf conf = FlinkUtils.toCelebornConf(configuration);
+    CelebornConf celebornConf = FlinkUtils.toCelebornConf(configuration);
 
-    long numBytesPerGate = conf.clientFlinkMemoryPerInputGate();
+    long numBytesPerGate = celebornConf.clientFlinkMemoryPerInputGate();
     long expectedInput = 2 * numBytesPerGate;
 
-    long numBytesPerResultPartition = conf.clientFlinkMemoryPerResultPartition();
+    long numBytesPerResultPartition = celebornConf.clientFlinkMemoryPerResultPartition();
     long expectedOutput = 3 * numBytesPerResultPartition;
     MemorySize expected = new MemorySize(expectedInput + expectedOutput);
 
@@ -253,11 +275,15 @@ public class RemoteShuffleMasterTest {
         intermediateResultPartitionId,
         ResultPartitionType.BLOCKING,
         5,
-        1);
+        1,
+        false,
+        false);
   }
 
   public ProducerDescriptor createProducerDescriptor() throws UnknownHostException {
-    ExecutionAttemptID executionAttemptId = new ExecutionAttemptID();
+    ExecutionAttemptID executionAttemptId =
+        new ExecutionAttemptID(
+            new ExecutionGraphID(), new ExecutionVertexID(new JobVertexID(0, 0), 0), 0);
     return new ProducerDescriptor(
         ResourceID.generate(), executionAttemptId, InetAddress.getLocalHost(), 100);
   }
