@@ -26,6 +26,7 @@ import org.apache.spark.*;
 import org.apache.spark.launcher.SparkLauncher;
 import org.apache.spark.shuffle.*;
 import org.apache.spark.shuffle.sort.SortShuffleManager;
+import org.apache.spark.sql.internal.SQLConf;
 import org.apache.spark.util.Utils;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
@@ -44,9 +45,6 @@ public class SparkShuffleManager implements ShuffleManager {
   private static final String SORT_SHUFFLE_MANAGER_NAME =
       "org.apache.spark.shuffle.sort.SortShuffleManager";
 
-  private static final String LOCAL_SHUFFLE_READER_KEY =
-      "spark.sql.adaptive.localShuffleReader.enabled";
-
   private final SparkConf conf;
   private final Boolean isDriver;
   private final CelebornConf celebornConf;
@@ -68,11 +66,11 @@ public class SparkShuffleManager implements ShuffleManager {
   private long sendBufferPoolExpireTimeout;
 
   public SparkShuffleManager(SparkConf conf, boolean isDriver) {
-    if (conf.getBoolean(LOCAL_SHUFFLE_READER_KEY, true)) {
+    if (conf.getBoolean(SQLConf.LOCAL_SHUFFLE_READER_ENABLED().key(), true)) {
       logger.warn(
           "Detected {} (default is true) is enabled, it's highly recommended to disable it when "
               + "use Celeborn as Remote Shuffle Service to avoid performance degradation.",
-          LOCAL_SHUFFLE_READER_KEY);
+          SQLConf.LOCAL_SHUFFLE_READER_ENABLED().key());
     }
     this.conf = conf;
     this.isDriver = isDriver;
@@ -129,7 +127,8 @@ public class SparkShuffleManager implements ShuffleManager {
 
     if (fallbackPolicyRunner.applyAllFallbackPolicy(
         lifecycleManager, dependency.partitioner().numPartitions())) {
-      if (conf.getBoolean("spark.dynamicAllocation.enabled", false)) {
+      if (conf.getBoolean("spark.dynamicAllocation.enabled", false)
+          && !conf.getBoolean("spark.shuffle.service.enabled", false)) {
         logger.error(
             "DRA is enabled but we fallback to vanilla Spark SortShuffleManager for "
                 + "shuffle: {} due to fallback policy. It may cause block can not found when reducer "
@@ -184,8 +183,8 @@ public class SparkShuffleManager implements ShuffleManager {
       lifecycleManager.stop();
       lifecycleManager = null;
     }
-    if (sortShuffleManager() != null) {
-      sortShuffleManager().stop();
+    if (_sortShuffleManager != null) {
+      _sortShuffleManager.stop();
       _sortShuffleManager = null;
     }
   }
