@@ -36,6 +36,7 @@ import org.slf4j.LoggerFactory;
 
 import org.apache.celeborn.client.compress.Compressor;
 import org.apache.celeborn.client.read.CelebornInputStream;
+import org.apache.celeborn.client.read.MetricsCallback;
 import org.apache.celeborn.common.CelebornConf;
 import org.apache.celeborn.common.exception.CelebornIOException;
 import org.apache.celeborn.common.identity.UserIdentifier;
@@ -72,8 +73,8 @@ public class ShuffleClientImpl extends ShuffleClient {
 
   private final int registerShuffleMaxRetries;
   private final long registerShuffleRetryWaitMs;
-  private int maxReviveTimes;
-  private boolean testRetryRevive;
+  private final int maxReviveTimes;
+  private final boolean testRetryRevive;
   private final int pushBufferMaxSize;
   protected final long pushDataTimeout;
 
@@ -112,7 +113,7 @@ public class ShuffleClientImpl extends ShuffleClient {
 
   protected final String appUniqueId;
 
-  private ThreadLocal<Compressor> compressorThreadLocal =
+  private final ThreadLocal<Compressor> compressorThreadLocal =
       new ThreadLocal<Compressor>() {
         @Override
         protected Compressor initialValue() {
@@ -171,8 +172,8 @@ public class ShuffleClientImpl extends ShuffleClient {
       pushDataTimeout = conf.pushDataTimeoutMs();
     }
 
-    // init rpc env and master endpointRef
-    rpcEnv = RpcEnv.create("ShuffleClient", Utils.localHostName(conf), 0, conf);
+    // init rpc env
+    rpcEnv = RpcEnv.create(RpcNameConstants.SHUFFLE_CLIENT_SYS, Utils.localHostName(conf), 0, conf);
 
     String module = TransportModuleConstants.DATA_MODULE;
     TransportConf dataTransportConf =
@@ -272,6 +273,7 @@ public class ShuffleClientImpl extends ShuffleClient {
           partitionId,
           batchId,
           newLoc);
+      pushDataRpcResponseCallback.updateLatestPartition(newLoc);
       try {
         if (!isPushTargetWorkerExcluded(newLoc, pushDataRpcResponseCallback)) {
           if (!testRetryRevive || remainReviveTimes < 1) {
@@ -281,7 +283,6 @@ public class ShuffleClientImpl extends ShuffleClient {
             String shuffleKey = Utils.makeShuffleKey(appUniqueId, shuffleId);
             PushData newPushData =
                 new PushData(PRIMARY_MODE, shuffleKey, newLoc.getUniqueId(), newBuffer);
-            pushDataRpcResponseCallback.updateLatestPartition(newLoc);
             client.pushData(newPushData, pushDataTimeout, pushDataRpcResponseCallback);
           } else {
             throw new RuntimeException(
@@ -600,13 +601,13 @@ public class ShuffleClientImpl extends ShuffleClient {
   }
 
   /**
-   * check if a newer PartitionLocation(with larger epoch) exists in local cache
+   * Check if a newer PartitionLocation(with larger epoch) exists in local cache.
    *
-   * @param shuffleMap
-   * @param partitionId
-   * @param epoch
-   * @param wait whether to wait for some time for a newer PartitionLocation
-   * @return
+   * @param shuffleMap The mapping between shuffle id and partition location.
+   * @param partitionId The id of partition.
+   * @param epoch The epoch of revive.
+   * @param wait Whether to wait for some time for a newer partition location.
+   * @return whether newer partition location exists in local cache.
    */
   boolean newerPartitionLocationExists(
       Map<Integer, PartitionLocation> shuffleMap, int partitionId, int epoch, boolean wait) {
@@ -633,18 +634,17 @@ public class ShuffleClientImpl extends ShuffleClient {
 
   void excludeWorkerByCause(StatusCode cause, PartitionLocation oldLocation) {
     if (pushExcludeWorkerOnFailureEnabled && oldLocation != null) {
-      if (cause == StatusCode.PUSH_DATA_CREATE_CONNECTION_FAIL_PRIMARY) {
-        pushExcludedWorkers.add(oldLocation.hostAndPushPort());
-      } else if (cause == StatusCode.PUSH_DATA_CONNECTION_EXCEPTION_PRIMARY) {
-        pushExcludedWorkers.add(oldLocation.hostAndPushPort());
-      } else if (cause == StatusCode.PUSH_DATA_TIMEOUT_PRIMARY) {
-        pushExcludedWorkers.add(oldLocation.hostAndPushPort());
-      } else if (cause == StatusCode.PUSH_DATA_CREATE_CONNECTION_FAIL_REPLICA) {
-        pushExcludedWorkers.add(oldLocation.getPeer().hostAndPushPort());
-      } else if (cause == StatusCode.PUSH_DATA_CONNECTION_EXCEPTION_REPLICA) {
-        pushExcludedWorkers.add(oldLocation.getPeer().hostAndPushPort());
-      } else if (cause == StatusCode.PUSH_DATA_TIMEOUT_REPLICA) {
-        pushExcludedWorkers.add(oldLocation.getPeer().hostAndPushPort());
+      switch (cause) {
+        case PUSH_DATA_CREATE_CONNECTION_FAIL_PRIMARY:
+        case PUSH_DATA_CONNECTION_EXCEPTION_PRIMARY:
+        case PUSH_DATA_TIMEOUT_PRIMARY:
+          pushExcludedWorkers.add(oldLocation.hostAndPushPort());
+          break;
+        case PUSH_DATA_CREATE_CONNECTION_FAIL_REPLICA:
+        case PUSH_DATA_CONNECTION_EXCEPTION_REPLICA:
+        case PUSH_DATA_TIMEOUT_REPLICA:
+          pushExcludedWorkers.add(oldLocation.getPeer().hostAndPushPort());
+          break;
       }
     }
   }
@@ -675,12 +675,10 @@ public class ShuffleClientImpl extends ShuffleClient {
           attemptId,
           partitionId);
       return true;
-    } else if (results == null
-        || !results.containsKey(partitionId)
-        || results.get(partitionId) != StatusCode.SUCCESS.getValue()) {
-      return false;
     } else {
-      return true;
+      return results != null
+          && results.containsKey(partitionId)
+          && results.get(partitionId) == StatusCode.SUCCESS.getValue();
     }
   }
 
@@ -876,8 +874,10 @@ public class ShuffleClientImpl extends ShuffleClient {
           new RpcResponseCallback() {
             @Override
             public void onSuccess(ByteBuffer response) {
-              if (response.remaining() > 0 && response.get() == StatusCode.STAGE_ENDED.getValue()) {
-                stageEndShuffleSet.add(shuffleId);
+              if (response.remaining() > 0 && response.get() == StatusCode.MAP_ENDED.getValue()) {
+                mapperEndMap
+                    .computeIfAbsent(shuffleId, (id) -> ConcurrentHashMap.newKeySet())
+                    .add(mapId);
               }
               logger.debug(
                   "Push data to {} success for shuffle {} map {} attempt {} partition {} batch {}.",
@@ -905,10 +905,10 @@ public class ShuffleClientImpl extends ShuffleClient {
             PartitionLocation latest = loc;
 
             @Override
-            public void updateLatestPartition(PartitionLocation latest) {
-              pushState.addBatch(nextBatchId, latest.hostAndPushPort());
+            public void updateLatestPartition(PartitionLocation newloc) {
+              pushState.addBatch(nextBatchId, newloc.hostAndPushPort());
               pushState.removeBatch(nextBatchId, this.latest.hostAndPushPort());
-              this.latest = latest;
+              this.latest = newloc;
             }
 
             @Override
@@ -1003,12 +1003,10 @@ public class ShuffleClientImpl extends ShuffleClient {
 
             @Override
             public void onFailure(Throwable e) {
-              StatusCode cause = getPushDataFailCause(e.getMessage());
-
               if (pushState.exception.get() != null) {
                 return;
               }
-
+              StatusCode cause = getPushDataFailCause(e.getMessage());
               if (remainReviveTimes <= 0) {
                 if (e instanceof CelebornIOException) {
                   callback.onFailure(e);
@@ -1273,8 +1271,10 @@ public class ShuffleClientImpl extends ShuffleClient {
                 groupedBatchId,
                 Arrays.toString(batchIds));
             pushState.removeBatch(groupedBatchId, hostPort);
-            if (response.remaining() > 0 && response.get() == StatusCode.STAGE_ENDED.getValue()) {
-              stageEndShuffleSet.add(shuffleId);
+            if (response.remaining() > 0 && response.get() == StatusCode.MAP_ENDED.getValue()) {
+              mapperEndMap
+                  .computeIfAbsent(shuffleId, (id) -> ConcurrentHashMap.newKeySet())
+                  .add(mapId);
             }
           }
 
@@ -1383,11 +1383,10 @@ public class ShuffleClientImpl extends ShuffleClient {
 
           @Override
           public void onFailure(Throwable e) {
-            StatusCode cause = getPushDataFailCause(e.getMessage());
-
             if (pushState.exception.get() != null) {
               return;
             }
+            StatusCode cause = getPushDataFailCause(e.getMessage());
             if (remainReviveTimes <= 0) {
               if (e instanceof CelebornIOException) {
                 callback.onFailure(e);
@@ -1527,12 +1526,6 @@ public class ShuffleClientImpl extends ShuffleClient {
     return true;
   }
 
-  @Override
-  public CelebornInputStream readPartition(int shuffleId, int partitionId, int attemptNumber)
-      throws IOException {
-    return readPartition(shuffleId, partitionId, attemptNumber, 0, Integer.MAX_VALUE);
-  }
-
   protected ReduceFileGroups loadFileGroupInternal(int shuffleId) {
     {
       long getReducerFileGroupStartTime = System.nanoTime();
@@ -1550,26 +1543,26 @@ public class ShuffleClientImpl extends ShuffleClient {
                 conf.clientRpcGetReducerFileGroupRpcAskTimeout(),
                 ClassTag$.MODULE$.apply(GetReducerFileGroupResponse.class));
 
-        if (response.status() == StatusCode.SUCCESS) {
-          logger.info(
-              "Shuffle {} request reducer file group success using {} ms, result partition size {}.",
-              shuffleId,
-              TimeUnit.NANOSECONDS.toMillis(System.nanoTime() - getReducerFileGroupStartTime),
-              response.fileGroup().size());
-          return new ReduceFileGroups(
-              response.fileGroup(), response.attempts(), response.partitionIds());
-        } else if (response.status() == StatusCode.STAGE_END_TIME_OUT) {
-          logger.warn(
-              "Request {} return {} for {}.",
-              getReducerFileGroup,
-              StatusCode.STAGE_END_TIME_OUT,
-              shuffleId);
-        } else if (response.status() == StatusCode.SHUFFLE_DATA_LOST) {
-          logger.warn(
-              "Request {} return {} for {}.",
-              getReducerFileGroup,
-              StatusCode.SHUFFLE_DATA_LOST,
-              shuffleId);
+        switch (response.status()) {
+          case SUCCESS:
+            logger.info(
+                "Shuffle {} request reducer file group success using {} ms, result partition size {}.",
+                shuffleId,
+                TimeUnit.NANOSECONDS.toMillis(System.nanoTime() - getReducerFileGroupStartTime),
+                response.fileGroup().size());
+            return new ReduceFileGroups(
+                response.fileGroup(), response.attempts(), response.partitionIds());
+          case STAGE_END_TIME_OUT:
+          case SHUFFLE_DATA_LOST:
+            logger.warn(
+                "Request {} return {} for {}.", getReducerFileGroup, response.status(), shuffleId);
+            return null;
+          case SHUFFLE_NOT_REGISTERED:
+            logger.warn(
+                "Request {} return {} for {}.", getReducerFileGroup, response.status(), shuffleId);
+            // return empty result
+            return new ReduceFileGroups(
+                response.fileGroup(), response.attempts(), response.partitionIds());
         }
       } catch (Exception e) {
         logger.error("Exception raised while call GetReducerFileGroup for {}.", shuffleId, e);
@@ -1595,11 +1588,16 @@ public class ShuffleClientImpl extends ShuffleClient {
 
   @Override
   public CelebornInputStream readPartition(
-      int shuffleId, int partitionId, int attemptNumber, int startMapIndex, int endMapIndex)
+      int shuffleId,
+      int partitionId,
+      int attemptNumber,
+      int startMapIndex,
+      int endMapIndex,
+      MetricsCallback metricsCallback)
       throws IOException {
     ReduceFileGroups fileGroups = loadFileGroup(shuffleId, partitionId);
 
-    if (fileGroups.partitionGroups.size() == 0
+    if (fileGroups.partitionGroups.isEmpty()
         || !fileGroups.partitionGroups.containsKey(partitionId)) {
       logger.warn("Shuffle data is empty for shuffle {} partition {}.", shuffleId, partitionId);
       return CelebornInputStream.empty();
@@ -1614,7 +1612,8 @@ public class ShuffleClientImpl extends ShuffleClient {
           attemptNumber,
           startMapIndex,
           endMapIndex,
-          fetchExcludedWorkers);
+          fetchExcludedWorkers,
+          metricsCallback);
     }
   }
 
@@ -1625,6 +1624,9 @@ public class ShuffleClientImpl extends ShuffleClient {
 
   @Override
   public void shutdown() {
+    if (null != reviveManager) {
+      reviveManager.close();
+    }
     if (null != rpcEnv) {
       rpcEnv.shutdown();
     }
@@ -1669,7 +1671,7 @@ public class ShuffleClientImpl extends ShuffleClient {
     logger.debug("Push data failed cause message: " + message);
     StatusCode cause;
     if (message == null) {
-      logger.error("Push data throw unexpected exception: {}", message);
+      logger.error("Push data throw unexpected exception");
       cause = StatusCode.PUSH_DATA_FAIL_NON_CRITICAL_CAUSE;
     } else if (message.startsWith(StatusCode.PUSH_DATA_WRITE_FAIL_REPLICA.name())) {
       cause = StatusCode.PUSH_DATA_WRITE_FAIL_REPLICA;
