@@ -56,7 +56,8 @@ public abstract class CelebornInputStream extends InputStream {
       int attemptNumber,
       int startMapIndex,
       int endMapIndex,
-      ConcurrentHashMap<String, Long> fetchExcludedWorkers)
+      ConcurrentHashMap<String, Long> fetchExcludedWorkers,
+      MetricsCallback metricsCallback)
       throws IOException {
     if (locations == null || locations.length == 0) {
       return emptyInputStream;
@@ -70,7 +71,8 @@ public abstract class CelebornInputStream extends InputStream {
           attemptNumber,
           startMapIndex,
           endMapIndex,
-          fetchExcludedWorkers);
+          fetchExcludedWorkers,
+          metricsCallback);
     }
   }
 
@@ -78,8 +80,6 @@ public abstract class CelebornInputStream extends InputStream {
     return emptyInputStream;
   }
 
-  public abstract void setCallback(MetricsCallback callback);
-
   private static final CelebornInputStream emptyInputStream =
       new CelebornInputStream() {
         @Override
@@ -91,9 +91,6 @@ public abstract class CelebornInputStream extends InputStream {
         public int read(byte[] b, int off, int len) throws IOException {
           return -1;
         }
-
-        @Override
-        public void setCallback(MetricsCallback callback) {}
       };
 
   private static final class CelebornInputStreamImpl extends CelebornInputStream {
@@ -139,6 +136,8 @@ public abstract class CelebornInputStream extends InputStream {
     private long fetchExcludedWorkerExpireTimeout;
     private final ConcurrentHashMap<String, Long> fetchExcludedWorkers;
 
+    private boolean containLocalRead = false;
+
     CelebornInputStreamImpl(
         CelebornConf conf,
         TransportClientFactory clientFactory,
@@ -148,7 +147,8 @@ public abstract class CelebornInputStream extends InputStream {
         int attemptNumber,
         int startMapIndex,
         int endMapIndex,
-        ConcurrentHashMap<String, Long> fetchExcludedWorkers)
+        ConcurrentHashMap<String, Long> fetchExcludedWorkers,
+        MetricsCallback metricsCallback)
         throws IOException {
       this.conf = conf;
       this.clientFactory = clientFactory;
@@ -186,6 +186,7 @@ public abstract class CelebornInputStream extends InputStream {
       TransportConf transportConf =
           Utils.fromCelebornConf(conf, TransportModuleConstants.DATA_MODULE, 0);
       retryWaitMs = transportConf.ioRetryWaitTimeMs();
+      this.callback = metricsCallback;
       moveToNextReader();
     }
 
@@ -297,6 +298,7 @@ public abstract class CelebornInputStream extends InputStream {
     }
 
     private PartitionReader createReaderWithRetry(PartitionLocation location) throws IOException {
+      Exception lastException = null;
       while (fetchChunkRetryCnt < fetchChunkMaxRetry) {
         try {
           if (isExcluded(location)) {
@@ -304,6 +306,7 @@ public abstract class CelebornInputStream extends InputStream {
           }
           return createReader(location, fetchChunkRetryCnt, fetchChunkMaxRetry);
         } catch (Exception e) {
+          lastException = e;
           excludeFailedLocation(location, e);
           fetchChunkRetryCnt++;
           if (location.hasPeer()) {
@@ -312,13 +315,13 @@ public abstract class CelebornInputStream extends InputStream {
             if (fetchChunkRetryCnt % 2 == 0) {
               Uninterruptibles.sleepUninterruptibly(retryWaitMs, TimeUnit.MILLISECONDS);
             }
-            location = location.getPeer();
             logger.warn(
                 "CreatePartitionReader failed {}/{} times for location {}, change to peer",
                 fetchChunkRetryCnt,
                 fetchChunkMaxRetry,
                 location,
                 e);
+            location = location.getPeer();
           } else {
             logger.warn(
                 "CreatePartitionReader failed {}/{} times for location {}, retry the same location",
@@ -330,7 +333,7 @@ public abstract class CelebornInputStream extends InputStream {
           }
         }
       }
-      throw new CelebornIOException("createPartitionReader failed! " + location);
+      throw new CelebornIOException("createPartitionReader failed! " + location, lastException);
     }
 
     private ByteBuf getNextChunk() throws IOException {
@@ -388,49 +391,40 @@ public abstract class CelebornInputStream extends InputStream {
         throws IOException, InterruptedException {
       if (!location.hasPeer()) {
         logger.debug("Partition {} has only one partition replica.", location);
-      }
-      if (location.hasPeer() && attemptNumber % 2 == 1) {
+      } else if (attemptNumber % 2 == 1) {
         location = location.getPeer();
         logger.debug("Read peer {} for attempt {}.", location, attemptNumber);
       }
-
       logger.debug("Create reader for location {}", location);
 
       StorageInfo storageInfo = location.getStorageInfo();
-      if (storageInfo.getType() == StorageInfo.Type.HDD
-          || storageInfo.getType() == StorageInfo.Type.SSD) {
-        logger.debug(
-            "Read local shuffle file enabled {} , {}, {}",
-            enabledReadLocalShuffle,
-            location.getWorker().host(),
-            localHostAddress);
-        if (enabledReadLocalShuffle && location.getWorker().host().equals(localHostAddress)) {
-          return new LocalPartitionReader(
-              conf, shuffleKey, location, clientFactory, startMapIndex, endMapIndex);
-        } else {
-          return new WorkerPartitionReader(
-              conf,
-              shuffleKey,
-              location,
-              clientFactory,
-              startMapIndex,
-              endMapIndex,
-              fetchChunkRetryCnt,
-              fetchChunkMaxRetry);
-        }
+      switch (storageInfo.getType()) {
+        case HDD:
+        case SSD:
+          if (enabledReadLocalShuffle && location.getWorker().host().equals(localHostAddress)) {
+            logger.debug("Read local shuffle file {}", localHostAddress);
+            containLocalRead = true;
+            return new LocalPartitionReader(
+                conf, shuffleKey, location, clientFactory, startMapIndex, endMapIndex, callback);
+          } else {
+            return new WorkerPartitionReader(
+                conf,
+                shuffleKey,
+                location,
+                clientFactory,
+                startMapIndex,
+                endMapIndex,
+                fetchChunkRetryCnt,
+                fetchChunkMaxRetry,
+                callback);
+          }
+        case HDFS:
+          return new DfsPartitionReader(
+              conf, shuffleKey, location, clientFactory, startMapIndex, endMapIndex, callback);
+        default:
+          throw new CelebornIOException(
+              String.format("Unknown storage info %s to read location %s", storageInfo, location));
       }
-      if (storageInfo.getType() == StorageInfo.Type.HDFS) {
-        return new DfsPartitionReader(
-            conf, shuffleKey, location, clientFactory, startMapIndex, endMapIndex);
-      }
-
-      throw new CelebornIOException(
-          "Unknown storage info " + storageInfo + " to read location " + location);
-    }
-
-    public void setCallback(MetricsCallback callback) {
-      // callback must set before read()
-      this.callback = callback;
     }
 
     @Override
@@ -499,8 +493,8 @@ public abstract class CelebornInputStream extends InputStream {
         currentReader.close();
         currentReader = null;
       }
-      if (enabledReadLocalShuffle) {
-        logger.info(ShuffleClient.getReadCounters());
+      if (containLocalRead) {
+        ShuffleClient.printReadStats(logger);
       }
     }
 
@@ -528,8 +522,6 @@ public abstract class CelebornInputStream extends InputStream {
         return false;
       }
 
-      long startTime = System.currentTimeMillis();
-
       boolean hasData = false;
       while (currentChunk.isReadable() || moveToNextChunk()) {
         currentChunk.readBytes(sizeBuf);
@@ -561,9 +553,7 @@ public abstract class CelebornInputStream extends InputStream {
           Set<Integer> batchSet = batchesRead.get(mapId);
           if (!batchSet.contains(batchId)) {
             batchSet.add(batchId);
-            if (callback != null) {
-              callback.incBytesRead(BATCH_HEADER_SIZE + size);
-            }
+            callback.incBytesRead(BATCH_HEADER_SIZE + size);
             if (shuffleCompressionEnabled) {
               // decompress data
               int originalLength = decompressor.getOriginalLen(compressedBuf);
@@ -587,9 +577,6 @@ public abstract class CelebornInputStream extends InputStream {
         }
       }
 
-      if (callback != null) {
-        callback.incReadTime(System.currentTimeMillis() - startTime);
-      }
       return hasData;
     }
   }
