@@ -22,13 +22,14 @@ import java.util.concurrent.{ConcurrentHashMap, ScheduledExecutorService, Schedu
 import java.util.concurrent.atomic.{AtomicInteger, LongAdder}
 
 import scala.collection.JavaConverters._
+import scala.collection.mutable.ArrayBuffer
 import scala.concurrent.duration.DurationInt
 
 import org.roaringbitmap.RoaringBitmap
 
 import org.apache.celeborn.client.CommitManager.CommittedPartitionInfo
 import org.apache.celeborn.client.LifecycleManager.ShuffleFailedWorkers
-import org.apache.celeborn.client.commit.{CommitHandler, MapPartitionCommitHandler, ReducePartitionCommitHandler}
+import org.apache.celeborn.client.commit.{CommitFilesParam, CommitHandler, MapPartitionCommitHandler, ReducePartitionCommitHandler}
 import org.apache.celeborn.client.listener.{WorkersStatus, WorkerStatusListener}
 import org.apache.celeborn.common.CelebornConf
 import org.apache.celeborn.common.internal.Logging
@@ -72,8 +73,9 @@ object CommitManager {
 class CommitManager(appUniqueId: String, val conf: CelebornConf, lifecycleManager: LifecycleManager)
   extends Logging {
 
+  // exposed for test
   // shuffle id -> ShuffleCommittedInfo
-  private val committedPartitionInfo = new CommittedPartitionInfo
+  val committedPartitionInfo = new CommittedPartitionInfo
   private val batchHandleCommitPartitionEnabled = conf.batchHandleCommitPartitionEnabled
   private val batchHandleCommitPartitionExecutors = ThreadUtils.newDaemonCachedThreadPool(
     "celeborn-lifecycle-manager-commit-partition-executor",
@@ -94,7 +96,7 @@ class CommitManager(appUniqueId: String, val conf: CelebornConf, lifecycleManage
     lifecycleManager.registerWorkerStatusListener(new ShutdownWorkerListener)
 
     batchHandleCommitPartition = batchHandleCommitPartitionSchedulerThread.map {
-      _.scheduleAtFixedRate(
+      _.scheduleWithFixedDelay(
         new Runnable {
           override def run(): Unit = {
             committedPartitionInfo.asScala.foreach { case (shuffleId, shuffleCommittedInfo) =>
@@ -113,13 +115,9 @@ class CommitManager(appUniqueId: String, val conf: CelebornConf, lifecycleManage
 
                     if (workerToRequests.nonEmpty) {
                       val commitFilesFailedWorkers = new ShuffleFailedWorkers()
-                      val parallelism =
-                        Math.min(workerToRequests.size, conf.clientRpcMaxParallelism)
                       try {
-                        ThreadUtils.parmap(
-                          workerToRequests,
-                          "CommitFiles",
-                          parallelism) {
+                        val params = new ArrayBuffer[CommitFilesParam](workerToRequests.size)
+                        workerToRequests.foreach {
                           case (worker, requests) =>
                             val workerInfo =
                               lifecycleManager.shuffleAllocatedWorkers
@@ -141,15 +139,18 @@ class CommitManager(appUniqueId: String, val conf: CelebornConf, lifecycleManage
                                 .toList
                                 .asJava
 
-                            commitHandler.commitFiles(
-                              appUniqueId,
-                              shuffleId,
-                              shuffleCommittedInfo,
+                            params += CommitFilesParam(
                               workerInfo,
                               primaryIds,
-                              replicaIds,
-                              commitFilesFailedWorkers)
+                              replicaIds)
                         }
+
+                        commitHandler.doParallelCommitFiles(
+                          shuffleId,
+                          shuffleCommittedInfo,
+                          params,
+                          commitFilesFailedWorkers)
+
                         lifecycleManager.workerStatusTracker.recordWorkerFailure(
                           commitFilesFailedWorkers)
                       } finally {
@@ -266,32 +267,31 @@ class CommitManager(appUniqueId: String, val conf: CelebornConf, lifecycleManage
     getCommitHandler(shuffleId).handleGetReducerFileGroup(context, shuffleId)
   }
 
-  private def getCommitHandler(shuffleId: Int): CommitHandler = {
+  // exposed for test
+  def getCommitHandler(shuffleId: Int): CommitHandler = {
     val partitionType = lifecycleManager.getPartitionType(shuffleId)
-    if (commitHandlers.containsKey(partitionType)) {
-      commitHandlers.get(partitionType)
-    } else {
-      commitHandlers.computeIfAbsent(
-        partitionType,
-        (partitionType: PartitionType) => {
-          partitionType match {
-            case PartitionType.REDUCE => new ReducePartitionCommitHandler(
-                appUniqueId,
-                conf,
-                lifecycleManager.shuffleAllocatedWorkers,
-                committedPartitionInfo,
-                lifecycleManager.workerStatusTracker)
-            case PartitionType.MAP => new MapPartitionCommitHandler(
-                appUniqueId,
-                conf,
-                lifecycleManager.shuffleAllocatedWorkers,
-                committedPartitionInfo,
-                lifecycleManager.workerStatusTracker)
-            case _ => throw new UnsupportedOperationException(
-                s"Unexpected ShufflePartitionType for CommitManager: $partitionType")
-          }
-        })
-    }
+    commitHandlers.computeIfAbsent(
+      partitionType,
+      (partitionType: PartitionType) => {
+        partitionType match {
+          case PartitionType.REDUCE => new ReducePartitionCommitHandler(
+              appUniqueId,
+              conf,
+              lifecycleManager.shuffleAllocatedWorkers,
+              committedPartitionInfo,
+              lifecycleManager.workerStatusTracker,
+              lifecycleManager.rpcSharedThreadPool)
+          case PartitionType.MAP => new MapPartitionCommitHandler(
+              appUniqueId,
+              conf,
+              lifecycleManager.shuffleAllocatedWorkers,
+              committedPartitionInfo,
+              lifecycleManager.workerStatusTracker,
+              lifecycleManager.rpcSharedThreadPool)
+          case _ => throw new UnsupportedOperationException(
+              s"Unexpected ShufflePartitionType for CommitManager: $partitionType")
+        }
+      })
   }
 
   def commitMetrics(): (Long, Long) = commitHandlers.asScala.values.foldLeft(0L -> 0L) {
