@@ -17,15 +17,22 @@
 
 package org.apache.celeborn.client
 
+import java.nio.ByteBuffer
 import java.util
 import java.util.{function, List => JList}
-import java.util.concurrent.{ConcurrentHashMap, ScheduledFuture, TimeUnit}
+import java.util.concurrent.{Callable, ConcurrentHashMap, LinkedBlockingQueue, ScheduledFuture, TimeUnit}
+import java.util.concurrent.atomic.AtomicInteger
+import java.util.function.Consumer
 
 import scala.collection.JavaConverters._
+import scala.collection.generic.CanBuildFrom
 import scala.collection.mutable
+import scala.concurrent.{ExecutionContext, Future}
+import scala.concurrent.duration.Duration
 import scala.util.Random
 
 import com.google.common.annotations.VisibleForTesting
+import com.google.common.cache.{Cache, CacheBuilder}
 
 import org.apache.celeborn.client.LifecycleManager.{ShuffleAllocatedWorkers, ShuffleFailedWorkers}
 import org.apache.celeborn.client.listener.WorkerStatusListener
@@ -39,9 +46,11 @@ import org.apache.celeborn.common.protocol.RpcNameConstants.WORKER_EP
 import org.apache.celeborn.common.protocol.message.ControlMessages._
 import org.apache.celeborn.common.protocol.message.StatusCode
 import org.apache.celeborn.common.rpc._
+import org.apache.celeborn.common.rpc.netty.{LocalNettyRpcCallContext, RemoteNettyRpcCallContext}
 import org.apache.celeborn.common.util.{JavaUtils, PbSerDeUtils, ThreadUtils, Utils}
 // Can Remove this if celeborn don't support scala211 in future
 import org.apache.celeborn.common.util.FunctionConverter._
+import org.apache.celeborn.common.util.ThreadUtils.awaitResult
 
 object LifecycleManager {
   // shuffle id -> partition id -> partition locations
@@ -75,6 +84,20 @@ class LifecycleManager(val appUniqueId: String, val conf: CelebornConf) extends
   val latestPartitionLocation =
     JavaUtils.newConcurrentHashMap[Int, ConcurrentHashMap[Int, PartitionLocation]]()
   private val userIdentifier: UserIdentifier = IdentityProvider.instantiate(conf).provide()
+  private val availableStorageTypes = conf.availableStorageTypes
+
+  private val rpcCacheSize = conf.clientRpcCacheSize
+  private val rpcCacheConcurrencyLevel = conf.clientRpcCacheConcurrencyLevel
+  private val rpcCacheExpireTime = conf.clientRpcCacheExpireTime
+  private val rpcMaxRetires = conf.clientRpcMaxRetries
+
+  private val registerShuffleResponseRpcCache: Cache[Int, ByteBuffer] = CacheBuilder.newBuilder()
+    .concurrencyLevel(rpcCacheConcurrencyLevel)
+    .expireAfterAccess(rpcCacheExpireTime, TimeUnit.MILLISECONDS)
+    .maximumSize(rpcCacheSize)
+    .build().asInstanceOf[Cache[Int, ByteBuffer]]
+
+  private val mockDestroyFailure = conf.testMockDestroySlotsFailure
 
   @VisibleForTesting
   def workerSnapshots(shuffleId: Int): util.Map[WorkerInfo, ShufflePartitionLocationInfo] =
@@ -108,6 +131,9 @@ class LifecycleManager(val appUniqueId: String, val conf: CelebornConf) extends
   private val forwardMessageThread =
     ThreadUtils.newDaemonSingleThreadScheduledExecutor("master-forward-message-thread")
   private var checkForShuffleRemoval: ScheduledFuture[_] = _
+  val rpcSharedThreadPool =
+    ThreadUtils.newDaemonCachedThreadPool("shared-rpc-pool", conf.clientRpcSharedThreads, 30)
+  val ec = ExecutionContext.fromExecutor(rpcSharedThreadPool)
 
   // init driver celeborn LifecycleManager rpc service
   override val rpcEnv: RpcEnv = RpcEnv.create(
@@ -147,7 +173,7 @@ class LifecycleManager(val appUniqueId: String, val conf: CelebornConf) extends
 
   override def onStart(): Unit = {
     // noinspection ConvertExpressionToSAM
-    checkForShuffleRemoval = forwardMessageThread.scheduleAtFixedRate(
+    checkForShuffleRemoval = forwardMessageThread.scheduleWithFixedDelay(
       new Runnable {
         override def run(): Unit = Utils.tryLogNonFatalError {
           self.send(RemoveExpiredShuffle)
@@ -293,11 +319,61 @@ class LifecycleManager(val appUniqueId: String, val conf: CelebornConf) extends
       }
 
     case GetReducerFileGroup(shuffleId: Int) =>
-      logDebug(s"Received GetShuffleFileGroup request," +
-        s"shuffleId $shuffleId.")
+      logDebug(s"Received GetShuffleFileGroup request for shuffleId $shuffleId.")
       handleGetReducerFileGroup(context, shuffleId)
   }
 
+  def setupEndpoints(
+      slots: WorkerResource,
+      shuffleId: Int,
+      connectFailedWorkers: ShuffleFailedWorkers): Unit = {
+    val futures = new util.LinkedList[(Future[RpcEndpointRef], WorkerInfo)]()
+    slots.asScala foreach { case (workerInfo, _) =>
+      val future = rpcEnv.asyncSetupEndpointRefByAddr(RpcEndpointAddress(
+        RpcAddress.apply(workerInfo.host, workerInfo.rpcPort),
+        WORKER_EP))
+      futures.add((future, workerInfo))
+    }
+
+    var timeout = conf.rpcAskTimeout.duration.toMillis
+    val delta = 50
+    while (timeout > 0 && !futures.isEmpty) {
+      val iter = futures.iterator
+      while (iter.hasNext) {
+        val (future, workerInfo) = iter.next()
+        if (future.isCompleted) {
+          future.value.get match {
+            case scala.util.Success(endpointRef) =>
+              workerInfo.endpoint = endpointRef
+            case scala.util.Failure(e) =>
+              logError(
+                s"Init rpc client failed for $shuffleId on $workerInfo during reserve slots.",
+                e)
+              connectFailedWorkers.put(
+                workerInfo,
+                (StatusCode.WORKER_UNKNOWN, System.currentTimeMillis()))
+          }
+          iter.remove()
+        }
+      }
+
+      if (!futures.isEmpty) {
+        Thread.sleep(delta)
+        timeout -= delta
+      }
+    }
+    if (!futures.isEmpty) {
+      val iter = futures.iterator()
+      while (iter.hasNext) {
+        val (_, workerInfo) = iter.next()
+        logError(s"Init rpc client failed for $shuffleId on $workerInfo during reserve slots, reason: Timeout.")
+        connectFailedWorkers.put(
+          workerInfo,
+          (StatusCode.WORKER_UNKNOWN, System.currentTimeMillis()))
+      }
+    }
+  }
+
   private def offerAndReserveSlots(
       context: RegisterCallContext,
       shuffleId: Int,
@@ -316,21 +392,32 @@ class LifecycleManager(val appUniqueId: String, val conf: CelebornConf) extends
         // If shuffle is registered, reply this shuffle's partition location and return.
         // Else add this request to registeringShuffleRequest.
         if (registeredShuffle.contains(shuffleId)) {
-          val initialLocs = workerSnapshots(shuffleId)
-            .values()
-            .asScala
-            .flatMap(_.getAllPrimaryLocationsWithMinEpoch().asScala)
-            .filter(p =>
-              (partitionType == PartitionType.REDUCE && p.getEpoch == 0) || (partitionType == PartitionType.MAP && p.getId == partitionId))
-            .toArray
+          val rpcContext: RpcCallContext = context.context
           partitionType match {
-            case PartitionType.MAP => processMapTaskReply(
+            case PartitionType.MAP =>
+              processMapTaskReply(
                 shuffleId,
-                context.context,
+                rpcContext,
                 partitionId,
-                initialLocs)
+                getInitialLocs(shuffleId, p => p.getId == partitionId))
             case PartitionType.REDUCE =>
-              context.reply(RegisterShuffleResponse(StatusCode.SUCCESS, initialLocs))
+              if (rpcContext.isInstanceOf[LocalNettyRpcCallContext]) {
+                context.reply(RegisterShuffleResponse(
+                  StatusCode.SUCCESS,
+                  getInitialLocs(shuffleId, p => p.getEpoch == 0)))
+              } else {
+                val cachedMsg = registerShuffleResponseRpcCache.get(
+                  shuffleId,
+                  new Callable[ByteBuffer]() {
+                    override def call(): ByteBuffer = {
+                      rpcContext.asInstanceOf[RemoteNettyRpcCallContext].nettyEnv.serialize(
+                        RegisterShuffleResponse(
+                          StatusCode.SUCCESS,
+                          getInitialLocs(shuffleId, p => p.getEpoch == 0)))
+                    }
+                  })
+                rpcContext.asInstanceOf[RemoteNettyRpcCallContext].callback.onSuccess(cachedMsg)
+              }
             case _ =>
               throw new UnsupportedOperationException(s"Not support $partitionType yet")
           }
@@ -345,6 +432,17 @@ class LifecycleManager(val appUniqueId: String, val conf: CelebornConf) extends
       }
     }
 
+    def getInitialLocs(
+        shuffleId: Int,
+        partitionLocationFilter: PartitionLocation => Boolean): Array[PartitionLocation] = {
+      workerSnapshots(shuffleId)
+        .values()
+        .asScala
+        .flatMap(_.getAllPrimaryLocationsWithMinEpoch())
+        .filter(partitionLocationFilter)
+        .toArray
+    }
+
     def processMapTaskReply(
         shuffleId: Int,
         context: RpcCallContext,
@@ -365,8 +463,24 @@ class LifecycleManager(val appUniqueId: String, val conf: CelebornConf) extends
     }
 
     // Reply to all RegisterShuffle request for current shuffle id.
-    def reply(response: PbRegisterShuffleResponse): Unit = {
+    def replyRegisterShuffle(response: PbRegisterShuffleResponse): Unit = {
       registeringShuffleRequest.synchronized {
+        val serializedMsg: Option[ByteBuffer] = partitionType match {
+          case PartitionType.REDUCE =>
+            context.context match {
+              case remoteContext: RemoteNettyRpcCallContext =>
+                if (response.getStatus == StatusCode.SUCCESS.getValue) {
+                  Option(remoteContext.nettyEnv.serialize(
+                    response))
+                } else {
+                  Option.empty
+                }
+
+              case _ => Option.empty
+            }
+          case _ => Option.empty
+        }
+
         registeringShuffleRequest.asScala
           .get(shuffleId)
           .foreach(_.asScala.foreach(context => {
@@ -387,7 +501,15 @@ class LifecycleManager(val appUniqueId: String, val conf: CelebornConf) extends
                   // otherwise will lost original exception message
                   context.reply(response)
                 }
-              case PartitionType.REDUCE => context.reply(response)
+              case PartitionType.REDUCE =>
+                if (context.context.isInstanceOf[
+                    LocalNettyRpcCallContext] || response.getStatus != StatusCode.SUCCESS.getValue) {
+                  context.reply(response)
+                } else {
+                  registerShuffleResponseRpcCache.put(shuffleId, serializedMsg.get)
+                  context.context.asInstanceOf[RemoteNettyRpcCallContext].callback.onSuccess(
+                    serializedMsg.get)
+                }
               case _ =>
                 throw new UnsupportedOperationException(s"Not support $partitionType yet")
             }
@@ -404,11 +526,11 @@ class LifecycleManager(val appUniqueId: String, val conf: CelebornConf) extends
     res.status match {
       case StatusCode.REQUEST_FAILED =>
         logInfo(s"OfferSlots RPC request failed for $shuffleId!")
-        reply(RegisterShuffleResponse(StatusCode.REQUEST_FAILED, Array.empty))
+        replyRegisterShuffle(RegisterShuffleResponse(StatusCode.REQUEST_FAILED, Array.empty))
         return
       case StatusCode.SLOT_NOT_AVAILABLE =>
         logInfo(s"OfferSlots for $shuffleId failed!")
-        reply(RegisterShuffleResponse(StatusCode.SLOT_NOT_AVAILABLE, Array.empty))
+        replyRegisterShuffle(RegisterShuffleResponse(StatusCode.SLOT_NOT_AVAILABLE, Array.empty))
         return
       case StatusCode.SUCCESS =>
         logInfo(s"OfferSlots for $shuffleId Success!Slots Info: ${res.workerResource}")
@@ -423,19 +545,7 @@ class LifecycleManager(val appUniqueId: String, val conf: CelebornConf) extends
     val connectFailedWorkers = new ShuffleFailedWorkers()
 
     // Second, for each worker, try to initialize the endpoint.
-    val parallelism = Math.min(Math.max(1, slots.size()), conf.clientRpcMaxParallelism)
-    ThreadUtils.parmap(slots.asScala, "InitWorkerRef", parallelism) { case (workerInfo, _) =>
-      try {
-        workerInfo.endpoint =
-          rpcEnv.setupEndpointRef(RpcAddress.apply(workerInfo.host, workerInfo.rpcPort), WORKER_EP)
-      } catch {
-        case t: Throwable =>
-          logError(s"Init rpc client failed for $shuffleId on $workerInfo during reserve slots.", t)
-          connectFailedWorkers.put(
-            workerInfo,
-            (StatusCode.WORKER_UNKNOWN, System.currentTimeMillis()))
-      }
-    }
+    setupEndpoints(slots, shuffleId, connectFailedWorkers)
 
     candidatesWorkers.removeAll(connectFailedWorkers.asScala.keys.toList.asJava)
     workerStatusTracker.recordWorkerFailure(connectFailedWorkers)
@@ -455,9 +565,11 @@ class LifecycleManager(val appUniqueId: String, val conf: CelebornConf) extends
     // If reserve slots failed, clear allocated resources, reply ReserveSlotFailed and return.
     if (!reserveSlotsSuccess) {
       logError(s"reserve buffer for $shuffleId failed, reply to all.")
-      reply(RegisterShuffleResponse(StatusCode.RESERVE_SLOTS_FAILED, Array.empty))
+      replyRegisterShuffle(RegisterShuffleResponse(StatusCode.RESERVE_SLOTS_FAILED, Array.empty))
     } else {
-      logInfo(s"ReserveSlots for $shuffleId success with details:$slots!")
+      if (log.isDebugEnabled()) {
+        logDebug(s"ReserveSlots for $shuffleId success with details:$slots!")
+      }
       // Forth, register shuffle success, update status
       val allocatedWorkers =
         JavaUtils.newConcurrentHashMap[WorkerInfo, ShufflePartitionLocationInfo]()
@@ -475,7 +587,9 @@ class LifecycleManager(val appUniqueId: String, val conf: CelebornConf) extends
       // Fifth, reply the allocated partition location to ShuffleClient.
       logInfo(s"Handle RegisterShuffle Success for $shuffleId.")
       val allPrimaryPartitionLocations = slots.asScala.flatMap(_._2._1.asScala).toArray
-      reply(RegisterShuffleResponse(StatusCode.SUCCESS, allPrimaryPartitionLocations))
+      replyRegisterShuffle(RegisterShuffleResponse(
+        StatusCode.SUCCESS,
+        allPrimaryPartitionLocations))
     }
   }
 
@@ -553,6 +667,14 @@ class LifecycleManager(val appUniqueId: String, val conf: CelebornConf) extends
   private def handleGetReducerFileGroup(
       context: RpcCallContext,
       shuffleId: Int): Unit = {
+    if (!registeredShuffle.contains(shuffleId)) {
+      logWarning(s"[handleGetReducerFileGroup] shuffle $shuffleId not registered, maybe no shuffle data within this stage.")
+      context.reply(GetReducerFileGroupResponse(
+        StatusCode.SHUFFLE_NOT_REGISTERED,
+        JavaUtils.newConcurrentHashMap(),
+        Array.empty))
+      return
+    }
     commitManager.handleGetReducerFileGroup(context, shuffleId)
   }
 
@@ -647,12 +769,12 @@ class LifecycleManager(val appUniqueId: String, val conf: CelebornConf) extends
     val reserveSlotFailedWorkers = new ShuffleFailedWorkers()
     val failureInfos = new util.concurrent.CopyOnWriteArrayList[String]()
     val workerPartitionLocations = slots.asScala.filter(p => !p._2._1.isEmpty || !p._2._2.isEmpty)
-    val parallelism =
-      Math.min(Math.max(1, workerPartitionLocations.size), conf.clientRpcMaxParallelism)
-    ThreadUtils.parmap(workerPartitionLocations, "ReserveSlot", parallelism) {
-      case (workerInfo, (primaryLocations, replicaLocations)) =>
-        val res = requestWorkerReserveSlots(
-          workerInfo.endpoint,
+
+    val (locsWithNullEndpoint, locs) = workerPartitionLocations.partition(_._1.endpoint == null)
+    val futures = new LinkedBlockingQueue[(Future[ReserveSlotsResponse], WorkerInfo)]()
+    val outFutures = locs.map { case (workerInfo, (primaryLocations, replicaLocations)) =>
+      Future {
+        val future = workerInfo.endpoint.ask[ReserveSlotsResponse](
           ReserveSlots(
             appUniqueId,
             shuffleId,
@@ -667,17 +789,74 @@ class LifecycleManager(val appUniqueId: String, val conf: CelebornConf) extends
             if (getPartitionType(shuffleId) == PartitionType.MAP)
               conf.clientShuffleMapPartitionSplitEnabled
             else true))
-        if (res.status.equals(StatusCode.SUCCESS)) {
-          logDebug(s"Successfully allocated " +
-            s"partitions buffer for shuffleId $shuffleId" +
-            s" from worker ${workerInfo.readableAddress()}.")
-        } else {
-          failureInfos.add(s"[reserveSlots] Failed to" +
-            s" reserve buffers for shuffleId $shuffleId" +
-            s" from worker ${workerInfo.readableAddress()}. Reason: ${res.reason}")
-          reserveSlotFailedWorkers.put(workerInfo, (res.status, System.currentTimeMillis()))
-        }
+        futures.add((future, workerInfo))
+      }(ec)
     }
+    val cbf =
+      implicitly[
+        CanBuildFrom[mutable.Iterable[Future[Boolean]], Boolean, mutable.Iterable[Boolean]]]
+    val futureSeq = Future.sequence(outFutures)(cbf, ec)
+    awaitResult(futureSeq, Duration.Inf)
+
+    var timeout = conf.rpcAskTimeout.duration.toMillis
+    val delta = 50
+    while (timeout >= 0 && !futures.isEmpty) {
+      val iter = futures.iterator()
+      while (iter.hasNext) {
+        val (future, workerInfo) = iter.next()
+        if (future.isCompleted) {
+          future.value.get match {
+            case scala.util.Success(res) =>
+              if (res.status.equals(StatusCode.SUCCESS)) {
+                logDebug(s"Successfully allocated " +
+                  s"partitions buffer for shuffleId $shuffleId" +
+                  s" from worker ${workerInfo.readableAddress()}.")
+              } else {
+                failureInfos.add(s"[reserveSlots] Failed to" +
+                  s" reserve buffers for shuffleId $shuffleId" +
+                  s" from worker ${workerInfo.readableAddress()}. Reason: ${res.reason}")
+                reserveSlotFailedWorkers.put(workerInfo, (res.status, System.currentTimeMillis()))
+              }
+            case scala.util.Failure(e) =>
+              failureInfos.add(s"[reserveSlots] Failed to" +
+                s" reserve buffers for shuffleId $shuffleId" +
+                s" from worker ${workerInfo.readableAddress()}. Reason: $e")
+              reserveSlotFailedWorkers.put(
+                workerInfo,
+                (StatusCode.REQUEST_FAILED, System.currentTimeMillis()))
+          }
+          iter.remove()
+        }
+      }
+
+      if (!futures.isEmpty) {
+        Thread.sleep(delta)
+      }
+      timeout = timeout - delta
+    }
+
+    val iter = futures.iterator()
+    while (iter.hasNext) {
+      val futureStatus = iter.next()
+      val workerInfo = futureStatus._2
+      failureInfos.add(s"[reserveSlots] Failed to" +
+        s" reserve buffers for shuffleId $shuffleId" +
+        s" from worker ${workerInfo.readableAddress()}. Reason: Timeout")
+      reserveSlotFailedWorkers.put(
+        workerInfo,
+        (StatusCode.REQUEST_FAILED, System.currentTimeMillis()))
+      iter.remove()
+    }
+
+    locsWithNullEndpoint.foreach { case (workerInfo, (_, _)) =>
+      failureInfos.add(s"[reserveSlots] Failed to" +
+        s" reserve buffers for shuffleId $shuffleId" +
+        s" from worker ${workerInfo.readableAddress()}. Reason: null endpoint")
+      reserveSlotFailedWorkers.put(
+        workerInfo,
+        (StatusCode.REQUEST_FAILED, System.currentTimeMillis()))
+    }
+
     if (failureInfos.asScala.nonEmpty) {
       logError(s"Aggregated error of reserveSlots for " +
         s"shuffleId $shuffleId " +
@@ -699,13 +878,14 @@ class LifecycleManager(val appUniqueId: String, val conf: CelebornConf) extends
    * @param slots                    allocated WorkerResource
    * @param failedPartitionLocations reserve slot failed partition location
    */
-  private def releasePeerPartitionLocation(
+  private def releasePartitionLocation(
       shuffleId: Int,
       slots: WorkerResource,
-      failedPartitionLocations: mutable.HashMap[Int, PartitionLocation]): Unit = {
+      failedPartitionLocations: mutable.HashMap[Int, PartitionLocation],
+      releasePeer: Boolean = false): Unit = {
     val destroyResource = new WorkerResource
     failedPartitionLocations.values
-      .flatMap { partition => Option(partition.getPeer) }
+      .flatMap { partition => if (releasePeer) Option(partition.getPeer) else Option(partition) }
       .foreach { partition =>
         var destroyWorkerInfo = partition.getWorker
         val workerInfoWithRpcRef = slots.keySet().asScala.find(_.equals(destroyWorkerInfo))
@@ -713,7 +893,7 @@ class LifecycleManager(val appUniqueId: String, val conf: CelebornConf) extends
             logWarning(s"Cannot find workInfo for $shuffleId from previous success workResource:" +
               s" ${destroyWorkerInfo.readableAddress()}, init according to partition info")
             try {
-              if (workerStatusTracker.workerAvailable(destroyWorkerInfo)) {
+              if (!workerStatusTracker.workerExcluded(destroyWorkerInfo)) {
                 destroyWorkerInfo.endpoint = rpcEnv.setupEndpointRef(
                   RpcAddress.apply(destroyWorkerInfo.host, destroyWorkerInfo.rpcPort),
                   WORKER_EP)
@@ -751,8 +931,11 @@ class LifecycleManager(val appUniqueId: String, val conf: CelebornConf) extends
       }
     if (!destroyResource.isEmpty) {
       destroySlotsWithRetry(shuffleId, destroyResource)
-      logInfo(s"Destroyed peer partitions for reserve buffer failed workers " +
-        s"shuffleId $shuffleId, $destroyResource")
+      val msg = destroyResource.asScala.map(entry =>
+        s"${entry._1.endpoint}, ${entry._2._1.asScala.map(
+          _.getUniqueId)}, ${entry._2._2.asScala.map(_.getUniqueId)}")
+      logWarning(s"Destroyed partitions for reserve buffer failed workers " +
+        s"shuffleId $shuffleId, $msg")
     }
   }
 
@@ -761,7 +944,7 @@ class LifecycleManager(val appUniqueId: String, val conf: CelebornConf) extends
    * and remove failed worker's partition locations from total slots.
    * For each reduce id, we only need to maintain one of the pair locations
    * even if enabling replicate. If Celeborn wants to release the failed partition location,
-   * the corresponding peers will be handled in [[releasePeerPartitionLocation]]
+   * the corresponding peers will be handled in [[releasePartitionLocation]]
    *
    * @param reserveFailedWorkers reserve slot failed WorkerInfo list of slots
    * @param slots                the slots tried to reserve a slot
@@ -772,7 +955,7 @@ class LifecycleManager(val appUniqueId: String, val conf: CelebornConf) extends
       slots: WorkerResource): mutable.HashMap[Int, PartitionLocation] = {
     val failedPartitionLocations = new mutable.HashMap[Int, PartitionLocation]()
     reserveFailedWorkers.asScala.foreach { workerInfo =>
-      val (failedPrimaryLocations, failedReplicaLocations) = slots.remove(workerInfo)
+      val (failedPrimaryLocations, failedReplicaLocations) = slots.get(workerInfo)
       if (null != failedPrimaryLocations) {
         failedPrimaryLocations.asScala.foreach { failedPrimaryLocation =>
           failedPartitionLocations += (failedPrimaryLocation.getId -> failedPrimaryLocation)
@@ -823,11 +1006,15 @@ class LifecycleManager(val appUniqueId: String, val conf: CelebornConf) extends
         candidates.removeAll(reserveFailedWorkers)
         // Find out all failed partition locations and remove failed worker's partition location
         // from slots.
-        val failedPartitionLocations = getFailedPartitionLocations(reserveFailedWorkers, slots)
+        val failedPartitionLocations =
+          getFailedPartitionLocations(reserveFailedWorkers, slots)
         // When enable replicate, if one of the partition location reserve slots failed, we also
         // need to release another corresponding partition location and remove it from slots.
+        if (failedPartitionLocations.nonEmpty && !slots.isEmpty) {
+          releasePartitionLocation(shuffleId, slots, failedPartitionLocations)
+        }
         if (pushReplicateEnabled && failedPartitionLocations.nonEmpty && !slots.isEmpty) {
-          releasePeerPartitionLocation(shuffleId, slots, failedPartitionLocations)
+          releasePartitionLocation(shuffleId, slots, failedPartitionLocations, true)
         }
         if (retryTimes < reserveSlotsMaxRetries) {
           // get retryCandidates resource and retry reserve buffer
@@ -867,7 +1054,7 @@ class LifecycleManager(val appUniqueId: String, val conf: CelebornConf) extends
     if (!success) {
       // Reserve slot failed workers' partition location and corresponding peer partition location
       // has been removed from slots by call [[getFailedPartitionLocations]] and
-      // [[releasePeerPartitionLocation]]. Now in the slots are all the successful partition
+      // [[releasePartitionLocation]]. Now in the slots are all the successful partition
       // locations.
       logWarning(s"Reserve buffers for $shuffleId still fail after retrying, clear buffers.")
       destroySlotsWithRetry(shuffleId, slots)
@@ -952,6 +1139,13 @@ class LifecycleManager(val appUniqueId: String, val conf: CelebornConf) extends
     slots
   }
 
+  case class DestroyFutureWithStatus(
+      var future: Future[DestroyWorkerSlotsResponse],
+      message: DestroyWorkerSlots,
+      endpoint: RpcEndpointRef,
+      var retryTimes: Int,
+      var startTime: Long)
+
   /**
    * For the slots that need to be destroyed, LifecycleManager will ask the corresponding worker
    * to destroy related FileWriter.
@@ -964,24 +1158,92 @@ class LifecycleManager(val appUniqueId: String, val conf: CelebornConf) extends
       shuffleId: Int,
       slotsToDestroy: WorkerResource): Unit = {
     val shuffleKey = Utils.makeShuffleKey(appUniqueId, shuffleId)
-    val parallelism = Math.min(Math.max(1, slotsToDestroy.size()), conf.clientRpcMaxParallelism)
-    ThreadUtils.parmap(
-      slotsToDestroy.asScala,
-      "DestroySlot",
-      parallelism) { case (workerInfo, (primaryLocations, replicaLocations)) =>
-      val destroy = DestroyWorkerSlots(
-        shuffleKey,
-        primaryLocations.asScala.map(_.getUniqueId).asJava,
-        replicaLocations.asScala.map(_.getUniqueId).asJava)
-      var res = requestWorkerDestroySlots(workerInfo.endpoint, destroy)
-      if (res.status != StatusCode.SUCCESS) {
-        logDebug(s"Request $destroy return ${res.status} for $shuffleKey, " +
-          s"will retry request destroy.")
-        res = requestWorkerDestroySlots(
-          workerInfo.endpoint,
-          DestroyWorkerSlots(shuffleKey, res.failedPrimarys, res.failedReplicas))
+
+    def retryDestroy(status: DestroyFutureWithStatus, currentTime: Long): Unit = {
+      status.retryTimes += 1
+      status.startTime = currentTime
+      // mock failure if mockDestroyFailure is true and this is not the last retry
+      status.message.mockFailure =
+        status.message.mockFailure && (status.retryTimes != rpcMaxRetires)
+      status.future =
+        status.endpoint.ask[DestroyWorkerSlotsResponse](status.message)
+    }
+
+    val startTime = System.currentTimeMillis()
+    val futures = new util.LinkedList[DestroyFutureWithStatus]()
+    slotsToDestroy.asScala.filter(_._1.endpoint != null).foreach {
+      case (workerInfo, (primaryLocations, replicaLocations)) =>
+        val primaryIds = primaryLocations.asScala.map(_.getUniqueId).asJava
+        val replicaIds = replicaLocations.asScala.map(_.getUniqueId).asJava
+        val destroy = DestroyWorkerSlots(shuffleKey, primaryIds, replicaIds, mockDestroyFailure)
+        val future = workerInfo.endpoint.ask[DestroyWorkerSlotsResponse](destroy)
+        futures.add(DestroyFutureWithStatus(future, destroy, workerInfo.endpoint, 1, startTime))
+    }
+
+    val timeout = conf.rpcAskTimeout.duration.toMillis
+    var remainingTime = timeout * rpcMaxRetires
+    val delta = 50
+    while (remainingTime > 0 && !futures.isEmpty) {
+      val currentTime = System.currentTimeMillis()
+      val iter = futures.iterator()
+      while (iter.hasNext) {
+        val futureWithStatus = iter.next()
+        val message = futureWithStatus.message
+        val retryTimes = futureWithStatus.retryTimes
+        if (futureWithStatus.future.isCompleted) {
+          futureWithStatus.future.value.get match {
+            case scala.util.Success(res) =>
+              if (res.status != StatusCode.SUCCESS && retryTimes < rpcMaxRetires) {
+                logError(
+                  s"Request $message to ${futureWithStatus.endpoint} return ${res.status} for $shuffleKey $retryTimes/$rpcMaxRetires, " +
+                    "will retry.")
+                retryDestroy(futureWithStatus, currentTime)
+              } else {
+                if (res.status != StatusCode.SUCCESS && retryTimes == rpcMaxRetires) {
+                  logError(
+                    s"Request $message to ${futureWithStatus.endpoint} return ${res.status} for $shuffleKey $retryTimes/$rpcMaxRetires, " +
+                      "will not retry.")
+                }
+                iter.remove()
+              }
+            case scala.util.Failure(e) =>
+              if (retryTimes < rpcMaxRetires) {
+                logError(
+                  s"Request $message to ${futureWithStatus.endpoint} failed $retryTimes/$rpcMaxRetires for $shuffleKey, reason: $e, " +
+                    "will retry.")
+                retryDestroy(futureWithStatus, currentTime)
+              } else {
+                if (retryTimes == rpcMaxRetires) {
+                  logError(
+                    s"Request $message to ${futureWithStatus.endpoint} failed $retryTimes/$rpcMaxRetires for $shuffleKey, reason: $e, " +
+                      "will not retry.")
+                }
+                iter.remove()
+              }
+          }
+        } else if (currentTime - futureWithStatus.startTime > timeout) {
+          if (retryTimes < rpcMaxRetires) {
+            logError(
+              s"Request $message to ${futureWithStatus.endpoint} failed $retryTimes/$rpcMaxRetires for $shuffleKey, reason: Timeout, " +
+                "will retry.")
+            retryDestroy(futureWithStatus, currentTime)
+          } else {
+            if (retryTimes == rpcMaxRetires) {
+              logError(
+                s"Request $message to ${futureWithStatus.endpoint} failed $retryTimes/$rpcMaxRetires for $shuffleKey, reason: Timeout, " +
+                  "will retry.")
+            }
+            iter.remove()
+          }
+        }
+      }
+
+      if (!futures.isEmpty) {
+        Thread.sleep(delta)
+        remainingTime -= delta
       }
     }
+    futures.clear()
   }
 
   private def removeExpiredShuffle(): Unit = {
@@ -1006,7 +1268,7 @@ class LifecycleManager(val appUniqueId: String, val conf: CelebornConf) extends
     }
   }
 
-  private def requestMasterRequestSlotsWithRetry(
+  def requestMasterRequestSlotsWithRetry(
       shuffleId: Int,
       ids: util.ArrayList[Integer]): RequestSlotsResponse = {
     val req =
@@ -1018,7 +1280,8 @@ class LifecycleManager(val appUniqueId: String, val conf: CelebornConf) extends
         pushReplicateEnabled,
         pushRackAwareEnabled,
         userIdentifier,
-        slotsAssignMaxWorkers)
+        slotsAssignMaxWorkers,
+        availableStorageTypes)
     val res = requestMasterRequestSlots(req)
     if (res.status != StatusCode.SUCCESS) {
       requestMasterRequestSlots(req)
@@ -1046,28 +1309,14 @@ class LifecycleManager(val appUniqueId: String, val conf: CelebornConf) extends
       endpoint.askSync[ReserveSlotsResponse](message, conf.clientRpcReserveSlotsRpcTimeout)
     } catch {
       case e: Exception =>
-        val msg = s"Exception when askSync ReserveSlots for $shuffleKey " +
-          s"on worker $endpoint."
+        val msg =
+          s"Exception when askSync worker(${endpoint.address}) ReserveSlots for $shuffleKey " +
+            s"on worker $endpoint."
         logError(msg, e)
         ReserveSlotsResponse(StatusCode.REQUEST_FAILED, msg + s" ${e.getMessage}")
     }
   }
 
-  private def requestWorkerDestroySlots(
-      endpoint: RpcEndpointRef,
-      message: DestroyWorkerSlots): DestroyWorkerSlotsResponse = {
-    try {
-      endpoint.askSync[DestroyWorkerSlotsResponse](message)
-    } catch {
-      case e: Exception =>
-        logError(s"AskSync Destroy for ${message.shuffleKey} failed.", e)
-        DestroyWorkerSlotsResponse(
-          StatusCode.REQUEST_FAILED,
-          message.primaryLocations,
-          message.replicaLocations)
-    }
-  }
-
   private def requestMasterUnregisterShuffle(message: PbUnregisterShuffle)
       : PbUnregisterShuffleResponse = {
     try {
@@ -1129,4 +1378,12 @@ class LifecycleManager(val appUniqueId: String, val conf: CelebornConf) extends
 
   // Initialize at the end of LifecycleManager construction.
   initialize()
+
+  /**
+   * A convenient method to stop [[RpcEndpoint]].
+   */
+  override def stop(): Unit = {
+    heartbeater.stop()
+    super.stop()
+  }
 }
