@@ -25,6 +25,7 @@ import scala.collection.JavaConverters._
 import org.junit.Assert
 
 import org.apache.celeborn.CelebornFunSuite
+import org.apache.celeborn.client.read.MetricsCallback
 import org.apache.celeborn.common.CelebornConf
 import org.apache.celeborn.common.identity.UserIdentifier
 import org.apache.celeborn.common.util.JavaUtils.timeOutOrMeetCondition
@@ -41,7 +42,12 @@ trait WithShuffleClientSuite extends CelebornFunSuite {
 
   private var lifecycleManager: LifecycleManager = _
   private var shuffleClient: ShuffleClientImpl = _
-  private var shuffleId = 0
+
+  var _shuffleId = 0
+  def nextShuffleId: Int = {
+    _shuffleId += 1
+    _shuffleId
+  }
 
   override protected def afterEach() {
     if (lifecycleManager != null) {
@@ -55,7 +61,7 @@ trait WithShuffleClientSuite extends CelebornFunSuite {
 
   test("test register map partition task") {
     prepareService()
-    shuffleId = 1
+    val shuffleId = nextShuffleId
     var location =
       shuffleClient.registerMapPartitionTask(shuffleId, numMappers, mapId, attemptId, 1)
     Assert.assertEquals(location.getId, 1)
@@ -94,10 +100,10 @@ trait WithShuffleClientSuite extends CelebornFunSuite {
   }
 
   test("test batch release partition") {
-    shuffleId = 2
+    val shuffleId = nextShuffleId
     celebornConf.set(CelebornConf.CLIENT_BATCH_HANDLE_RELEASE_PARTITION_ENABLED.key, "true")
     prepareService()
-    registerAndFinishPartition()
+    registerAndFinishPartition(shuffleId)
 
     val partitionLocationInfos = lifecycleManager.workerSnapshots(shuffleId).values().asScala
 
@@ -114,11 +120,11 @@ trait WithShuffleClientSuite extends CelebornFunSuite {
   }
 
   test("test release single partition") {
-    shuffleId = 3
+    val shuffleId = nextShuffleId
     celebornConf.set(CelebornConf.CLIENT_BATCH_HANDLE_RELEASE_PARTITION_ENABLED.key, "false")
     celebornConf.set(CelebornConf.CLIENT_BATCH_HANDLED_RELEASE_PARTITION_INTERVAL.key, "1s")
     prepareService()
-    registerAndFinishPartition()
+    registerAndFinishPartition(shuffleId)
 
     val partitionLocationInfos = lifecycleManager.workerSnapshots(shuffleId).values().asScala
 
@@ -133,19 +139,24 @@ trait WithShuffleClientSuite extends CelebornFunSuite {
   }
 
   test("test map end & get reducer file group") {
-    shuffleId = 4
+    val shuffleId = nextShuffleId
     prepareService()
-    registerAndFinishPartition()
+    registerAndFinishPartition(shuffleId)
 
     // reduce file group size (for empty partitions)
     Assert.assertEquals(shuffleClient.getReduceFileGroupsMap.size(), 0)
 
+    val metricsCallback = new MetricsCallback {
+      override def incBytesRead(bytesWritten: Long): Unit = {}
+      override def incReadTime(time: Long): Unit = {}
+    }
+
     // reduce normal empty CelebornInputStream
-    var stream = shuffleClient.readPartition(shuffleId, 1, 1)
+    var stream = shuffleClient.readPartition(shuffleId, 1, 1, 0, Integer.MAX_VALUE, metricsCallback)
     Assert.assertEquals(stream.read(), -1)
 
     // reduce normal null partition for CelebornInputStream
-    stream = shuffleClient.readPartition(shuffleId, 3, 1)
+    stream = shuffleClient.readPartition(shuffleId, 3, 1, 0, Integer.MAX_VALUE, metricsCallback)
     Assert.assertEquals(stream.read(), -1)
   }
 
@@ -155,7 +166,7 @@ trait WithShuffleClientSuite extends CelebornFunSuite {
     shuffleClient.setupLifecycleManagerRef(lifecycleManager.self)
   }
 
-  private def registerAndFinishPartition(): Unit = {
+  private def registerAndFinishPartition(shuffleId: Int): Unit = {
     shuffleClient.registerMapPartitionTask(shuffleId, numMappers, mapId, attemptId, 1)
     shuffleClient.registerMapPartitionTask(shuffleId, numMappers, mapId + 1, attemptId, 2)
     shuffleClient.registerMapPartitionTask(shuffleId, numMappers, mapId + 2, attemptId, 3)
