@@ -38,7 +38,7 @@ public class StorageInfo implements Serializable {
     }
   }
 
-  public static String UNKNOWN_DISK = "UNKNOWN_DISK";
+  @Deprecated public static String UNKNOWN_DISK = "UNKNOWN_DISK";
   public static Map<Integer, Type> typesMap = new HashMap<>();
   public static Set<String> typeNames = new HashSet<>();
 
@@ -49,6 +49,12 @@ public class StorageInfo implements Serializable {
     }
   }
 
+  public static final int MEMORY_MASK = 0b1;
+  public static final int LOCAL_DISK_MASK = 0b10;
+  public static final int HDFS_MASK = 0b100;
+  public static final int OSS_MASK = 0b1000;
+  public static final int ALL_TYPES_AVAILABLE_MASK = 0;
+
   // Default storage Type is MEMORY.
   private Type type = Type.MEMORY;
   private String mountPoint = UNKNOWN_DISK;
@@ -56,6 +62,8 @@ public class StorageInfo implements Serializable {
   private boolean finalResult = false;
   private String filePath;
 
+  public int availableStorageTypes = 0;
+
   public StorageInfo() {}
 
   public StorageInfo(Type type, boolean isFinal, String filePath) {
@@ -64,8 +72,9 @@ public class StorageInfo implements Serializable {
     this.filePath = filePath;
   }
 
-  public StorageInfo(String mountPoint) {
+  public StorageInfo(String mountPoint, int availableStorageTypes) {
     this.mountPoint = mountPoint;
+    this.availableStorageTypes = availableStorageTypes;
   }
 
   public StorageInfo(Type type, String mountPoint) {
@@ -86,6 +95,19 @@ public class StorageInfo implements Serializable {
     this.filePath = filePath;
   }
 
+  public StorageInfo(
+      Type type,
+      String mountPoint,
+      boolean finalResult,
+      String filePath,
+      int availableStorageTypes) {
+    this.type = type;
+    this.mountPoint = mountPoint;
+    this.finalResult = finalResult;
+    this.filePath = filePath;
+    this.availableStorageTypes = availableStorageTypes;
+  }
+
   public boolean isFinalResult() {
     return finalResult;
   }
@@ -125,13 +147,50 @@ public class StorageInfo implements Serializable {
         + '}';
   }
 
+  public boolean localDiskAvailable() {
+    return availableStorageTypes == ALL_TYPES_AVAILABLE_MASK
+        || (availableStorageTypes & LOCAL_DISK_MASK) > 0;
+  }
+
+  public boolean HDFSAvailable() {
+    return availableStorageTypes == ALL_TYPES_AVAILABLE_MASK
+        || (availableStorageTypes & HDFS_MASK) > 0;
+  }
+
+  public boolean OSSAvailable() {
+    return availableStorageTypes == ALL_TYPES_AVAILABLE_MASK
+        || (availableStorageTypes & OSS_MASK) > 0;
+  }
+
+  @Override
+  public boolean equals(Object o) {
+    if (this == o) return true;
+    if (o == null || getClass() != o.getClass()) return false;
+    StorageInfo that = (StorageInfo) o;
+    return finalResult == that.finalResult
+        && availableStorageTypes == that.availableStorageTypes
+        && type == that.type
+        && Objects.equals(mountPoint, that.mountPoint)
+        && Objects.equals(filePath, that.filePath);
+  }
+
+  @Override
+  public int hashCode() {
+    return Objects.hash(type, mountPoint, finalResult, filePath, availableStorageTypes);
+  }
+
+  public static final boolean validate(String typeStr) {
+    return typeNames.contains(typeStr);
+  }
+
   public static PbStorageInfo toPb(StorageInfo storageInfo) {
     String filePath = storageInfo.getFilePath();
     PbStorageInfo.Builder builder = PbStorageInfo.newBuilder();
     builder
         .setType(storageInfo.type.value)
         .setFinalResult(storageInfo.finalResult)
-        .setMountPoint(storageInfo.mountPoint);
+        .setMountPoint(storageInfo.mountPoint)
+        .setAvailableStorageTypes(storageInfo.availableStorageTypes);
     if (filePath != null) {
       builder.setFilePath(filePath);
     }
@@ -143,10 +202,29 @@ public class StorageInfo implements Serializable {
         typesMap.get(pbStorageInfo.getType()),
         pbStorageInfo.getMountPoint(),
         pbStorageInfo.getFinalResult(),
-        pbStorageInfo.getFilePath());
+        pbStorageInfo.getFilePath(),
+        pbStorageInfo.getAvailableStorageTypes());
   }
 
-  public static boolean validateStorageType(String str) {
-    return typeNames.contains(str);
+  public static int getAvailableTypes(List<Type> types) {
+    int ava = 0;
+    for (Type type : types) {
+      switch (type) {
+        case MEMORY:
+          ava = ava | MEMORY_MASK;
+          break;
+        case HDD:
+        case SSD:
+          ava = ava | LOCAL_DISK_MASK;
+          break;
+        case HDFS:
+          ava = ava | HDFS_MASK;
+          break;
+        case OSS:
+          ava = ava | OSS_MASK;
+          break;
+      }
+    }
+    return ava;
   }
 }
