@@ -170,6 +170,7 @@ object ControlMessages extends Logging {
       shouldRackAware: Boolean,
       userIdentifier: UserIdentifier,
       maxWorkers: Int,
+      availableStorageTypes: Int,
       override var requestId: String = ZERO_UUID)
     extends MasterRequestMessage
 
@@ -418,7 +419,8 @@ object ControlMessages extends Logging {
       primaryIds: util.List[String],
       replicaIds: util.List[String],
       mapAttempts: Array[Int],
-      epoch: Long)
+      epoch: Long,
+      var mockFailure: Boolean = false)
     extends WorkerMessage
 
   case class CommitFilesResponse(
@@ -439,7 +441,8 @@ object ControlMessages extends Logging {
   case class DestroyWorkerSlots(
       shuffleKey: String,
       primaryLocations: util.List[String],
-      replicaLocations: util.List[String])
+      replicaLocations: util.List[String],
+      var mockFailure: Boolean = false)
     extends WorkerMessage
 
   case class DestroyWorkerSlotsResponse(
@@ -526,6 +529,7 @@ object ControlMessages extends Logging {
           shouldRackAware,
           userIdentifier,
           maxWorkers,
+          availableStorageTypes,
           requestId) =>
       val payload = PbRequestSlots.newBuilder()
         .setApplicationId(applicationId)
@@ -536,6 +540,7 @@ object ControlMessages extends Logging {
         .setShouldRackAware(shouldRackAware)
         .setMaxWorkers(maxWorkers)
         .setRequestId(requestId)
+        .setAvailableStorageTypes(availableStorageTypes)
         .setUserIdentifier(PbSerDeUtils.toPbUserIdentifier(userIdentifier))
         .build().toByteArray
       new TransportMessage(MessageType.REQUEST_SLOTS, payload)
@@ -763,7 +768,14 @@ object ControlMessages extends Logging {
         .build().toByteArray
       new TransportMessage(MessageType.RESERVE_SLOTS_RESPONSE, payload)
 
-    case CommitFiles(applicationId, shuffleId, primaryIds, replicaIds, mapAttempts, epoch) =>
+    case CommitFiles(
+          applicationId,
+          shuffleId,
+          primaryIds,
+          replicaIds,
+          mapAttempts,
+          epoch,
+          mockFailure) =>
       val payload = PbCommitFiles.newBuilder()
         .setApplicationId(applicationId)
         .setShuffleId(shuffleId)
@@ -771,6 +783,7 @@ object ControlMessages extends Logging {
         .addAllReplicaIds(replicaIds)
         .addAllMapAttempts(mapAttempts.map(Integer.valueOf).toIterable.asJava)
         .setEpoch(epoch)
+        .setMockFailure(mockFailure)
         .build().toByteArray
       new TransportMessage(MessageType.COMMIT_FILES, payload)
 
@@ -803,11 +816,12 @@ object ControlMessages extends Logging {
       val payload = builder.build().toByteArray
       new TransportMessage(MessageType.COMMIT_FILES_RESPONSE, payload)
 
-    case DestroyWorkerSlots(shuffleKey, primaryLocations, replicaLocations) =>
+    case DestroyWorkerSlots(shuffleKey, primaryLocations, replicaLocations, mockFailure) =>
       val payload = PbDestroyWorkerSlots.newBuilder()
         .setShuffleKey(shuffleKey)
         .addAllPrimaryLocations(primaryLocations)
         .addAllReplicaLocation(replicaLocations)
+        .setMockFailure(mockFailure)
         .build().toByteArray
       new TransportMessage(MessageType.DESTROY, payload)
 
@@ -922,6 +936,7 @@ object ControlMessages extends Logging {
           pbRequestSlots.getShouldRackAware,
           userIdentifier,
           pbRequestSlots.getMaxWorkers,
+          pbRequestSlots.getAvailableStorageTypes,
           pbRequestSlots.getRequestId)
 
       case REQUEST_SLOTS_RESPONSE_VALUE =>
@@ -1083,7 +1098,8 @@ object ControlMessages extends Logging {
           pbCommitFiles.getPrimaryIdsList,
           pbCommitFiles.getReplicaIdsList,
           pbCommitFiles.getMapAttemptsList.asScala.map(_.toInt).toArray,
-          pbCommitFiles.getEpoch)
+          pbCommitFiles.getEpoch,
+          pbCommitFiles.getMockFailure)
 
       case COMMIT_FILES_RESPONSE_VALUE =>
         val pbCommitFilesResponse = PbCommitFilesResponse.parseFrom(message.getPayload)
@@ -1114,7 +1130,8 @@ object ControlMessages extends Logging {
         DestroyWorkerSlots(
           pbDestroy.getShuffleKey,
           pbDestroy.getPrimaryLocationsList,
-          pbDestroy.getReplicaLocationList)
+          pbDestroy.getReplicaLocationList,
+          pbDestroy.getMockFailure)
 
       case DESTROY_RESPONSE_VALUE =>
         val pbDestroyResponse = PbDestroyWorkerSlotsResponse.parseFrom(message.getPayload)
