@@ -477,14 +477,14 @@ object Utils extends Logging {
     hostPortParseResults.get(hostPort)
   }
 
-  private val MAX_DEFAULT_NETTY_THREADS = 64
+  private var maxDefaultNettyThreads = 64
 
   def fromCelebornConf(
       _conf: CelebornConf,
       module: String,
       numUsableCores: Int = 0): TransportConf = {
     val conf = _conf.clone
-
+    maxDefaultNettyThreads = conf.maxDefaultNettyThreads
     // Specify thread configuration based on our JVM's allocation of cores (rather than necessarily
     // assuming we have all the machine's cores).
     // NB: Only set if serverThreads/clientThreads not already set.
@@ -501,7 +501,7 @@ object Utils extends Logging {
   private def defaultNumThreads(numUsableCores: Int): Int = {
     val availableCores =
       if (numUsableCores > 0) numUsableCores else Runtime.getRuntime.availableProcessors()
-    math.min(availableCores, MAX_DEFAULT_NETTY_THREADS)
+    math.min(availableCores, maxDefaultNettyThreads)
   }
 
   def getClassLoader: ClassLoader = getClass.getClassLoader
@@ -816,7 +816,7 @@ object Utils extends Logging {
     val slotDistributions = new util.HashMap[String, Integer]()
     (masterLocations.asScala ++ workerLocations.asScala)
       .foreach {
-        case location =>
+        location =>
           val mountPoint = location.getStorageInfo.getMountPoint
           if (slotDistributions.containsKey(mountPoint)) {
             slotDistributions.put(mountPoint, slotDistributions.get(mountPoint) + 1)
@@ -824,10 +824,10 @@ object Utils extends Logging {
             slotDistributions.put(mountPoint, 1)
           }
       }
-    logDebug(s"locations to distribution ," +
-      s" ${masterLocations.asScala.map(_.toString).mkString(",")} " +
+    logDebug(s"locations to distribution, " +
+      s"${masterLocations.asScala.map(_.toString).mkString(",")} " +
       s"${workerLocations.asScala.map(_.toString).mkString(",")} " +
-      s"to ${slotDistributions} ")
+      s"to $slotDistributions ")
     slotDistributions
   }
 
@@ -971,6 +971,10 @@ object Utils extends Logging {
         StatusCode.FETCH_DATA_TIMEOUT
       case 47 =>
         StatusCode.REVIVE_INITIALIZED
+      case 48 =>
+        StatusCode.DESTROY_SLOTS_MOCK_FAILURE
+      case 49 =>
+        StatusCode.COMMIT_FILES_MOCK_FAILURE
       case _ =>
         null
     }
