@@ -19,6 +19,7 @@ package org.apache.celeborn.common
 
 import org.apache.celeborn.CelebornFunSuite
 import org.apache.celeborn.common.CelebornConf._
+import org.apache.celeborn.common.protocol.StorageInfo
 import org.apache.celeborn.common.util.Utils
 
 class CelebornConfSuite extends CelebornFunSuite {
@@ -193,11 +194,11 @@ class CelebornConfSuite extends CelebornFunSuite {
     conf.set("celeborn.storage.hdfs.dir", "hdfs:///xxx")
     assert(conf.workerBaseDirs.isEmpty)
 
-    conf.set("celeborn.storage.activeTypes", "SDD,HDD,HDFS")
+    conf.set("celeborn.storage.activeTypes", "SSD,HDD,HDFS")
     conf.set("celeborn.storage.hdfs.dir", "hdfs:///xxx")
     assert(conf.workerBaseDirs.isEmpty)
 
-    conf.set("celeborn.storage.activeTypes", "SDD,HDD")
+    conf.set("celeborn.storage.activeTypes", "SSD,HDD")
     assert(!conf.workerBaseDirs.isEmpty)
   }
 
@@ -207,7 +208,45 @@ class CelebornConfSuite extends CelebornFunSuite {
     conf.set("celeborn.storage.hdfs.dir", "hdfs:///xxx")
     assert(conf.workerCommitThreads === 128)
 
-    conf.set("celeborn.storage.activeTypes", "SDD,HDD")
+    conf.set("celeborn.storage.activeTypes", "SSD,HDD")
     assert(conf.workerCommitThreads === 32)
   }
+
+  test("Test available storage types") {
+    val conf = new CelebornConf()
+
+    assert(conf.availableStorageTypes == StorageInfo.LOCAL_DISK_MASK)
+
+    conf.set("celeborn.storage.availableTypes", "HDD,MEMORY")
+    assert(conf.availableStorageTypes == Integer.parseInt("11", 2))
+
+    conf.set("celeborn.storage.availableTypes", "HDD,HDFS")
+    assert(conf.availableStorageTypes == (StorageInfo.HDFS_MASK | StorageInfo.LOCAL_DISK_MASK))
+
+    conf.set("celeborn.storage.availableTypes", "HDFS")
+    assert(conf.availableStorageTypes == StorageInfo.HDFS_MASK)
+  }
+
+  test("Test role rpcDispatcherNumThreads") {
+    val availableCores = 5
+    val conf = new CelebornConf()
+    assert(conf.rpcDispatcherNumThreads(availableCores, "shuffleclient") === 5)
+
+    conf.set("celeborn.shuffleclient.rpc.dispatcher.threads", "1")
+    assert(conf.rpcDispatcherNumThreads(availableCores, "shuffleclient") === 1)
+    assert(conf.rpcDispatcherNumThreads(availableCores, "lifecyclemanager") === 5)
+
+    conf.set("celeborn.rpc.dispatcher.threads", "2")
+    assert(conf.rpcDispatcherNumThreads(availableCores, "lifecyclemanager") === 2)
+
+    conf.unset("celeborn.rpc.dispatcher.threads")
+    conf.set("celeborn.rpc.dispatcher.numThreads", "3")
+    assert(conf.rpcDispatcherNumThreads(availableCores, "lifecyclemanager") === 3)
+  }
+
+  test("Test rpcDispatcherNumThreads") {
+    val availableCores = 5
+    val conf = new CelebornConf()
+    assert(conf.rpcDispatcherNumThreads(availableCores) === 5)
+  }
 }
