@@ -19,6 +19,8 @@ package org.apache.celeborn.service.deploy.master;
 
 import java.util.*;
 
+import scala.Double;
+import scala.Option;
 import scala.Tuple2;
 
 import org.apache.commons.lang3.StringUtils;
@@ -31,6 +33,7 @@ import org.apache.celeborn.common.meta.DiskStatus;
 import org.apache.celeborn.common.meta.WorkerInfo;
 import org.apache.celeborn.common.protocol.PartitionLocation;
 import org.apache.celeborn.common.protocol.StorageInfo;
+import org.apache.celeborn.common.util.DiskUtils;
 
 public class SlotsAllocator {
   static class UsableDiskInfo {
@@ -53,15 +56,14 @@ public class SlotsAllocator {
           List<WorkerInfo> workers,
           List<Integer> partitionIds,
           boolean shouldReplicate,
-          boolean shouldRackAware) {
+          boolean shouldRackAware,
+          int availableStorageTypes) {
     if (partitionIds.isEmpty()) {
       return new HashMap<>();
     }
     if (workers.size() < 2 && shouldReplicate) {
       return new HashMap<>();
     }
-    Map<WorkerInfo, Tuple2<List<PartitionLocation>, List<PartitionLocation>>> slots =
-        new HashMap<>();
     Map<WorkerInfo, List<UsableDiskInfo>> restrictions = new HashMap<>();
     for (WorkerInfo worker : workers) {
       List<UsableDiskInfo> usableDisks =
@@ -74,16 +76,13 @@ public class SlotsAllocator {
         }
       }
     }
-    List<Integer> remain =
-        roundRobin(slots, partitionIds, workers, restrictions, shouldReplicate, shouldRackAware);
-    if (!remain.isEmpty()) {
-      remain = roundRobin(slots, remain, workers, null, shouldReplicate, shouldRackAware);
-    }
-
-    if (!remain.isEmpty()) {
-      roundRobin(slots, remain, workers, null, shouldReplicate, false);
-    }
-    return slots;
+    return locateSlots(
+        partitionIds,
+        workers,
+        restrictions,
+        shouldReplicate,
+        shouldRackAware,
+        availableStorageTypes);
   }
 
   /**
@@ -97,11 +96,13 @@ public class SlotsAllocator {
           List<Integer> partitionIds,
           boolean shouldReplicate,
           boolean shouldRackAware,
-          long minimumUsableSize,
+          long diskReserveSize,
+          Option<Double> diskReserveRatio,
           int diskGroupCount,
           double diskGroupGradient,
           double flushTimeWeight,
-          double fetchTimeWeight) {
+          double fetchTimeWeight,
+          int availableStorageTypes) {
     if (partitionIds.isEmpty()) {
       return new HashMap<>();
     }
@@ -118,64 +119,58 @@ public class SlotsAllocator {
                 .forEach(
                     (key, diskInfo) -> {
                       diskToWorkerMap.put(diskInfo, i);
-                      if (diskInfo.actualUsableSpace() > minimumUsableSize
+                      if (diskInfo.actualUsableSpace()
+                              > DiskUtils.getMinimumUsableSize(
+                                  diskInfo,
+                                  diskReserveSize,
+                                  diskReserveRatio.isEmpty()
+                                      ? Option.empty()
+                                      : Option.apply(diskReserveRatio.get()))
                           && diskInfo.status().equals(DiskStatus.HEALTHY)) {
                         usableDisks.add(diskInfo);
                       }
                     }));
 
-    Set<WorkerInfo> usableWorkers = new HashSet<>();
-    for (DiskInfo disk : usableDisks) {
-      usableWorkers.add(diskToWorkerMap.get(disk));
-    }
-    if ((shouldReplicate && usableWorkers.size() <= 1) || usableDisks.isEmpty()) {
+    boolean noUsableDisks =
+        usableDisks.isEmpty()
+            || (shouldReplicate
+                && (usableDisks.size() == 1
+                    || usableDisks.stream().map(diskToWorkerMap::get).distinct().count() <= 1));
+    boolean noAvailableSlots = usableDisks.stream().mapToLong(DiskInfo::availableSlots).sum() <= 0;
+
+    if (noUsableDisks || noAvailableSlots) {
       logger.warn(
-          "offer slots for {} fallback to roundrobin because there is no usable disks",
-          StringUtils.join(partitionIds, ','));
-      return offerSlotsRoundRobin(workers, partitionIds, shouldReplicate, shouldRackAware);
+          "offer slots for {} fallback to roundrobin because there is no {}",
+          StringUtils.join(partitionIds, ','),
+          noUsableDisks ? "usable disks" : "available slots");
+      return offerSlotsRoundRobin(
+          workers, partitionIds, shouldReplicate, shouldRackAware, availableStorageTypes);
     }
 
     if (!initialized) {
       initLoadAwareAlgorithm(diskGroupCount, diskGroupGradient);
     }
 
-    Map<WorkerInfo, List<UsableDiskInfo>> restriction =
+    Map<WorkerInfo, List<UsableDiskInfo>> restrictions =
         getRestriction(
             placeDisksToGroups(usableDisks, diskGroupCount, flushTimeWeight, fetchTimeWeight),
             diskToWorkerMap,
             shouldReplicate ? partitionIds.size() * 2 : partitionIds.size());
-
-    Map<WorkerInfo, Tuple2<List<PartitionLocation>, List<PartitionLocation>>> slots =
-        new HashMap<>();
-    List<Integer> remainPartitions =
-        roundRobin(
-            slots,
-            partitionIds,
-            new ArrayList<>(restriction.keySet()),
-            restriction,
-            shouldReplicate,
-            shouldRackAware);
-    if (!remainPartitions.isEmpty()) {
-      remainPartitions =
-          roundRobin(
-              slots,
-              remainPartitions,
-              new ArrayList<>(workers),
-              null,
-              shouldReplicate,
-              shouldRackAware);
-    }
-    if (!remainPartitions.isEmpty()) {
-      roundRobin(slots, remainPartitions, new ArrayList<>(workers), null, shouldReplicate, false);
-    }
-    return slots;
+    return locateSlots(
+        partitionIds,
+        workers,
+        restrictions,
+        shouldReplicate,
+        shouldRackAware,
+        availableStorageTypes);
   }
 
   private static StorageInfo getStorageInfo(
       List<WorkerInfo> workers,
       int workerIndex,
       Map<WorkerInfo, List<UsableDiskInfo>> restrictions,
-      Map<WorkerInfo, Integer> workerDiskIndex) {
+      Map<WorkerInfo, Integer> workerDiskIndex,
+      int availableStorageTypes) {
     WorkerInfo selectedWorker = workers.get(workerIndex);
     List<UsableDiskInfo> usableDiskInfos = restrictions.get(selectedWorker);
     int diskIndex = workerDiskIndex.computeIfAbsent(selectedWorker, v -> 0);
@@ -183,18 +178,58 @@ public class SlotsAllocator {
       diskIndex = (diskIndex + 1) % usableDiskInfos.size();
     }
     usableDiskInfos.get(diskIndex).usableSlots--;
-    StorageInfo storageInfo = new StorageInfo(usableDiskInfos.get(diskIndex).diskInfo.mountPoint());
+    StorageInfo storageInfo =
+        new StorageInfo(
+            usableDiskInfos.get(diskIndex).diskInfo.mountPoint(), availableStorageTypes);
     workerDiskIndex.put(selectedWorker, (diskIndex + 1) % usableDiskInfos.size());
     return storageInfo;
   }
 
+  /**
+   * Progressive locate slots for all partitions <br>
+   * 1. try to allocate for all partitions under restrictions <br>
+   * 2. allocate remain partitions to all workers <br>
+   * 3. allocate remain partitions to all workers again without considering rack aware <br>
+   */
+  private static Map<WorkerInfo, Tuple2<List<PartitionLocation>, List<PartitionLocation>>>
+      locateSlots(
+          List<Integer> partitionIds,
+          List<WorkerInfo> workers,
+          Map<WorkerInfo, List<UsableDiskInfo>> restrictions,
+          boolean shouldReplicate,
+          boolean shouldRackAware,
+          int activeStorageTypes) {
+    Map<WorkerInfo, Tuple2<List<PartitionLocation>, List<PartitionLocation>>> slots =
+        new HashMap<>();
+
+    List<Integer> remain =
+        roundRobin(
+            slots,
+            partitionIds,
+            new LinkedList<>(restrictions.keySet()),
+            restrictions,
+            shouldReplicate,
+            shouldRackAware,
+            activeStorageTypes);
+    if (!remain.isEmpty()) {
+      remain =
+          roundRobin(
+              slots, remain, workers, null, shouldReplicate, shouldRackAware, activeStorageTypes);
+    }
+    if (!remain.isEmpty()) {
+      roundRobin(slots, remain, workers, null, shouldReplicate, false, activeStorageTypes);
+    }
+    return slots;
+  }
+
   private static List<Integer> roundRobin(
       Map<WorkerInfo, Tuple2<List<PartitionLocation>, List<PartitionLocation>>> slots,
       List<Integer> partitionIds,
       List<WorkerInfo> workers,
       Map<WorkerInfo, List<UsableDiskInfo>> restrictions,
       boolean shouldReplicate,
-      boolean shouldRackAware) {
+      boolean shouldRackAware,
+      int availableStorageTypes) {
     // workerInfo -> (diskIndexForPrimary, diskIndexForReplica)
     Map<WorkerInfo, Integer> workerDiskIndexForPrimary = new HashMap<>();
     Map<WorkerInfo, Integer> workerDiskIndexForReplica = new HashMap<>();
@@ -203,39 +238,49 @@ public class SlotsAllocator {
     Iterator<Integer> iter = partitionIdList.iterator();
     outer:
     while (iter.hasNext()) {
-      int nextPrimayInd = primaryIndex;
+      int nextPrimaryInd = primaryIndex;
 
       int partitionId = iter.next();
       StorageInfo storageInfo = new StorageInfo();
       if (restrictions != null) {
-        while (!haveUsableSlots(restrictions, workers, nextPrimayInd)) {
-          nextPrimayInd = (nextPrimayInd + 1) % workers.size();
-          if (nextPrimayInd == primaryIndex) {
+        while (!haveUsableSlots(restrictions, workers, nextPrimaryInd)) {
+          nextPrimaryInd = (nextPrimaryInd + 1) % workers.size();
+          if (nextPrimaryInd == primaryIndex) {
             break outer;
           }
         }
         storageInfo =
-            getStorageInfo(workers, nextPrimayInd, restrictions, workerDiskIndexForPrimary);
+            getStorageInfo(
+                workers,
+                nextPrimaryInd,
+                restrictions,
+                workerDiskIndexForPrimary,
+                availableStorageTypes);
       }
       PartitionLocation primaryPartition =
-          createLocation(partitionId, workers.get(nextPrimayInd), null, storageInfo, true);
+          createLocation(partitionId, workers.get(nextPrimaryInd), null, storageInfo, true);
 
       if (shouldReplicate) {
-        int nextReplicaInd = (nextPrimayInd + 1) % workers.size();
+        int nextReplicaInd = (nextPrimaryInd + 1) % workers.size();
         if (restrictions != null) {
           while (!haveUsableSlots(restrictions, workers, nextReplicaInd)
-              || !satisfyRackAware(shouldRackAware, workers, nextPrimayInd, nextReplicaInd)) {
+              || !satisfyRackAware(shouldRackAware, workers, nextPrimaryInd, nextReplicaInd)) {
             nextReplicaInd = (nextReplicaInd + 1) % workers.size();
-            if (nextReplicaInd == nextPrimayInd) {
+            if (nextReplicaInd == nextPrimaryInd) {
               break outer;
             }
           }
           storageInfo =
-              getStorageInfo(workers, nextReplicaInd, restrictions, workerDiskIndexForReplica);
+              getStorageInfo(
+                  workers,
+                  nextReplicaInd,
+                  restrictions,
+                  workerDiskIndexForReplica,
+                  availableStorageTypes);
         } else if (shouldRackAware) {
-          while (!satisfyRackAware(true, workers, nextPrimayInd, nextReplicaInd)) {
+          while (!satisfyRackAware(true, workers, nextPrimaryInd, nextReplicaInd)) {
             nextReplicaInd = (nextReplicaInd + 1) % workers.size();
-            if (nextReplicaInd == nextPrimayInd) {
+            if (nextReplicaInd == nextPrimaryInd) {
               break outer;
             }
           }
@@ -253,9 +298,9 @@ public class SlotsAllocator {
 
       Tuple2<List<PartitionLocation>, List<PartitionLocation>> locations =
           slots.computeIfAbsent(
-              workers.get(nextPrimayInd), v -> new Tuple2<>(new ArrayList<>(), new ArrayList<>()));
+              workers.get(nextPrimaryInd), v -> new Tuple2<>(new ArrayList<>(), new ArrayList<>()));
       locations._1.add(primaryPartition);
-      primaryIndex = (nextPrimayInd + 1) % workers.size();
+      primaryIndex = (nextPrimaryInd + 1) % workers.size();
       iter.remove();
     }
     return partitionIdList;
@@ -460,10 +505,13 @@ public class SlotsAllocator {
       jointLocations.addAll(slots.get(worker)._2);
       for (PartitionLocation location : jointLocations) {
         String mountPoint = location.getStorageInfo().getMountPoint();
-        if (slotsPerDisk.containsKey(mountPoint)) {
-          slotsPerDisk.put(mountPoint, slotsPerDisk.get(mountPoint) + 1);
-        } else {
-          slotsPerDisk.put(mountPoint, 1);
+        // ignore slots for UNKNOWN_DISK
+        if (!mountPoint.equals(StorageInfo.UNKNOWN_DISK)) {
+          if (slotsPerDisk.containsKey(mountPoint)) {
+            slotsPerDisk.put(mountPoint, slotsPerDisk.get(mountPoint) + 1);
+          } else {
+            slotsPerDisk.put(mountPoint, 1);
+          }
         }
       }
     }
