@@ -53,15 +53,6 @@ public class SingleMasterMetaManager extends AbstractMetaManager {
     updateRequestSlotsMeta(shuffleKey, hostName, workerToAllocatedSlots);
   }
 
-  @Override
-  public void handleReleaseSlots(
-      String shuffleKey,
-      List<String> workerIds,
-      List<Map<String, Integer>> slotStrings,
-      String requestId) {
-    updateReleaseSlotsMeta(shuffleKey, workerIds, slotStrings);
-  }
-
   @Override
   public void handleUnRegisterShuffle(String shuffleKey, String requestId) {
     updateUnregisterShuffleMeta(shuffleKey);
@@ -90,6 +81,12 @@ public class SingleMasterMetaManager extends AbstractMetaManager {
     updateWorkerRemoveMeta(host, rpcPort, pushPort, fetchPort, replicatePort);
   }
 
+  @Override
+  public void handleRemoveWorkersUnavailableInfo(
+      List<WorkerInfo> unavailableWorkers, String requestId) {
+    removeWorkersUnavailableInfoMeta(unavailableWorkers);
+  }
+
   @Override
   public void handleWorkerHeartbeat(
       String host,
@@ -101,6 +98,7 @@ public class SingleMasterMetaManager extends AbstractMetaManager {
       Map<UserIdentifier, ResourceConsumption> userResourceConsumption,
       Map<String, Long> estimatedAppDiskUsage,
       long time,
+      boolean highWorkload,
       String requestId) {
     updateWorkerHeartbeatMeta(
         host,
@@ -111,7 +109,8 @@ public class SingleMasterMetaManager extends AbstractMetaManager {
         disks,
         userResourceConsumption,
         estimatedAppDiskUsage,
-        time);
+        time,
+        highWorkload);
   }
 
   @Override
