@@ -85,35 +85,6 @@ public class HAMasterMetaManager extends AbstractMetaManager {
     }
   }
 
-  @Override
-  public void handleReleaseSlots(
-      String shuffleKey,
-      List<String> workerIds,
-      List<Map<String, Integer>> slots,
-      String requestId) {
-    try {
-      ratisServer.submitRequest(
-          ResourceRequest.newBuilder()
-              .setCmdType(Type.ReleaseSlots)
-              .setRequestId(requestId)
-              .setReleaseSlotsRequest(
-                  ResourceProtos.ReleaseSlotsRequest.newBuilder()
-                      .setShuffleKey(shuffleKey)
-                      .addAllWorkerIds(workerIds)
-                      .addAllSlots(
-                          slots.stream()
-                              .map(
-                                  slot ->
-                                      ResourceProtos.SlotInfo.newBuilder().putAllSlot(slot).build())
-                              .collect(Collectors.toList()))
-                      .build())
-              .build());
-    } catch (CelebornRuntimeException e) {
-      LOG.error("Handle release slots for {} failed!", shuffleKey, e);
-      throw e;
-    }
-  }
-
   @Override
   public void handleUnRegisterShuffle(String shuffleKey, String requestId) {
     try {
@@ -215,6 +186,27 @@ public class HAMasterMetaManager extends AbstractMetaManager {
     }
   }
 
+  @Override
+  public void handleRemoveWorkersUnavailableInfo(
+      List<WorkerInfo> unavailableWorkers, String requestId) {
+    try {
+      List<ResourceProtos.WorkerAddress> addrs =
+          unavailableWorkers.stream().map(MetaUtil::infoToAddr).collect(Collectors.toList());
+      ratisServer.submitRequest(
+          ResourceRequest.newBuilder()
+              .setCmdType(Type.RemoveWorkersUnavailableInfo)
+              .setRequestId(requestId)
+              .setRemoveWorkersUnavailableInfoRequest(
+                  ResourceProtos.RemoveWorkersUnavailableInfoRequest.newBuilder()
+                      .addAllUnavailable(addrs)
+                      .build())
+              .build());
+    } catch (CelebornRuntimeException e) {
+      LOG.error("Handle remove workers unavailable info failed!", e);
+      throw e;
+    }
+  }
+
   @Override
   public void handleWorkerHeartbeat(
       String host,
@@ -226,6 +218,7 @@ public class HAMasterMetaManager extends AbstractMetaManager {
       Map<UserIdentifier, ResourceConsumption> userResourceConsumption,
       Map<String, Long> estimatedAppDiskUsage,
       long time,
+      boolean highWorkload,
       String requestId) {
     try {
       ratisServer.submitRequest(
@@ -244,6 +237,7 @@ public class HAMasterMetaManager extends AbstractMetaManager {
                           MetaUtil.toPbUserResourceConsumption(userResourceConsumption))
                       .putAllEstimatedAppDiskUsage(estimatedAppDiskUsage)
                       .setTime(time)
+                      .setHighWorkload(highWorkload)
                       .build())
               .build());
     } catch (CelebornRuntimeException e) {
