@@ -74,7 +74,7 @@ public class MetaHandler {
           throw new IOException("Can not parse this command!" + request);
       }
     } catch (IOException e) {
-      LOG.warn("Handle meta read request " + cmdType + " failed!", e);
+      LOG.warn("Handle meta read request {} failed!", cmdType, e);
       responseBuilder.setSuccess(false);
       responseBuilder.setStatus(ResourceProtos.Status.INTERNAL_ERROR);
       if (e.getMessage() != null) {
@@ -163,6 +163,7 @@ public class MetaHandler {
           estimatedAppDiskUsage.putAll(
               request.getWorkerHeartbeatRequest().getEstimatedAppDiskUsageMap());
           replicatePort = request.getWorkerHeartbeatRequest().getReplicatePort();
+          boolean highWorkload = request.getWorkerHeartbeatRequest().getHighWorkload();
           LOG.debug(
               "Handle worker heartbeat for {} {} {} {} {} {} {}",
               host,
@@ -182,7 +183,8 @@ public class MetaHandler {
               diskInfos,
               userResourceConsumption,
               estimatedAppDiskUsage,
-              time);
+              time,
+              highWorkload);
           break;
 
         case RegisterWorker:
@@ -226,12 +228,20 @@ public class MetaHandler {
           metaSystem.updatePartitionSize();
           break;
 
+        case RemoveWorkersUnavailableInfo:
+          List<ResourceProtos.WorkerAddress> unavailableList =
+              request.getRemoveWorkersUnavailableInfoRequest().getUnavailableList();
+          List<WorkerInfo> unavailableWorkers =
+              unavailableList.stream().map(MetaUtil::addrToInfo).collect(Collectors.toList());
+          metaSystem.removeWorkersUnavailableInfoMeta(unavailableWorkers);
+          break;
+
         default:
           throw new IOException("Can not parse this command!" + request);
       }
       responseBuilder.setStatus(ResourceProtos.Status.OK);
     } catch (IOException e) {
-      LOG.warn("Handle meta write request " + cmdType + " failed!", e);
+      LOG.warn("Handle meta write request {} failed!", cmdType, e);
       responseBuilder.setSuccess(false);
       responseBuilder.setStatus(ResourceProtos.Status.INTERNAL_ERROR);
       if (e.getMessage() != null) {
