@@ -27,6 +27,7 @@ import java.util.Map;
 import java.util.Random;
 import java.util.Set;
 
+import scala.Option;
 import scala.Tuple2;
 
 import org.junit.Assert;
@@ -36,6 +37,7 @@ import org.apache.celeborn.common.CelebornConf;
 import org.apache.celeborn.common.meta.DiskInfo;
 import org.apache.celeborn.common.meta.WorkerInfo;
 import org.apache.celeborn.common.protocol.PartitionLocation;
+import org.apache.celeborn.common.protocol.StorageInfo;
 
 public class SlotsAllocatorSuiteJ {
   private List<WorkerInfo> prepareWorkers(boolean hasDisks) {
@@ -236,10 +238,12 @@ public class SlotsAllocatorSuiteJ {
             shouldReplicate,
             false,
             10 * 1024 * 1024 * 1024L,
+            Option.empty(),
             conf.masterSlotAssignLoadAwareDiskGroupNum(),
             conf.masterSlotAssignLoadAwareDiskGroupGradient(),
             conf.masterSlotAssignLoadAwareFlushTimeWeight(),
-            conf.masterSlotAssignLoadAwareFetchTimeWeight());
+            conf.masterSlotAssignLoadAwareFetchTimeWeight(),
+            StorageInfo.ALL_TYPES_AVAILABLE_MASK);
     if (expectSuccess) {
       if (shouldReplicate) {
         slots.forEach(
@@ -275,10 +279,11 @@ public class SlotsAllocatorSuiteJ {
         allocateToDiskSlots += worker.usedSlots();
       }
       if (shouldReplicate) {
-        Assert.assertEquals(partitionIds.size() * 2, unknownDiskSlots + allocateToDiskSlots);
+        Assert.assertTrue(partitionIds.size() * 2 >= unknownDiskSlots + allocateToDiskSlots);
       } else {
-        Assert.assertEquals(partitionIds.size(), unknownDiskSlots + allocateToDiskSlots);
+        Assert.assertTrue(partitionIds.size() >= unknownDiskSlots + allocateToDiskSlots);
       }
+      Assert.assertEquals(0, unknownDiskSlots);
     } else {
       assert slots.isEmpty()
           : "Expect to fail to offer slots, but return " + slots.size() + " slots.";
@@ -294,7 +299,8 @@ public class SlotsAllocatorSuiteJ {
     CelebornConf conf = new CelebornConf();
     conf.set("celeborn.active.storage.levels", "HDFS");
     Map<WorkerInfo, Tuple2<List<PartitionLocation>, List<PartitionLocation>>> slots =
-        SlotsAllocator.offerSlotsRoundRobin(workers, partitionIds, shouldReplicate, false);
+        SlotsAllocator.offerSlotsRoundRobin(
+            workers, partitionIds, shouldReplicate, false, StorageInfo.ALL_TYPES_AVAILABLE_MASK);
 
     int allocatedPartitionCount = 0;
 
@@ -328,4 +334,16 @@ public class SlotsAllocatorSuiteJ {
     final boolean shouldReplicate = true;
     checkSlotsOnHDFS(workers, partitionIds, shouldReplicate, true);
   }
+
+  @Test
+  public void testAllocateSlotsWithNoAvailableSlots() {
+    final List<WorkerInfo> workers = prepareWorkers(true);
+    // Simulates no available slots behavior with greatly changed estimatedPartitionSize for workers
+    // with usable disks.
+    workers.forEach(workerInfo -> workerInfo.updateDiskMaxSlots(Long.MAX_VALUE));
+    final List<Integer> partitionIds = Collections.singletonList(0);
+    final boolean shouldReplicate = false;
+
+    check(workers, partitionIds, shouldReplicate, true);
+  }
 }
