@@ -19,21 +19,11 @@ package org.apache.celeborn.service.deploy.master.clustermeta.ha;
 
 import java.io.File;
 import java.io.IOException;
-import java.util.ArrayList;
-import java.util.Collections;
 import java.util.HashMap;
-import java.util.List;
 import java.util.Map;
 import java.util.UUID;
-import java.util.concurrent.ThreadLocalRandom;
-import java.util.regex.Matcher;
 
-import org.apache.ratis.server.storage.RaftStorage;
-import org.apache.ratis.server.storage.StorageImplUtils;
 import org.apache.ratis.statemachine.SnapshotInfo;
-import org.apache.ratis.statemachine.SnapshotRetentionPolicy;
-import org.apache.ratis.statemachine.impl.SimpleStateMachineStorage;
-import org.apache.ratis.statemachine.impl.SimpleStateMachineStorageUtil;
 import org.junit.Assert;
 import org.junit.Test;
 
@@ -44,7 +34,6 @@ import org.apache.celeborn.common.meta.DiskInfo;
 import org.apache.celeborn.common.meta.WorkerInfo;
 import org.apache.celeborn.common.quota.ResourceConsumption;
 import org.apache.celeborn.common.util.JavaUtils;
-import org.apache.celeborn.common.util.Utils;
 import org.apache.celeborn.service.deploy.master.clustermeta.ResourceProtos;
 import org.apache.celeborn.service.deploy.master.clustermeta.ResourceProtos.RequestSlotsRequest;
 import org.apache.celeborn.service.deploy.master.clustermeta.ResourceProtos.ResourceRequest;
@@ -114,93 +103,6 @@ public class MasterStateMachineSuiteJ extends RatisBaseSuiteJ {
     Assert.assertEquals(1, latest.getFiles().size());
   }
 
-  @Test
-  public void testSnapshotCleanup() throws IOException {
-    StateMachine stateMachine = ratisServer.getMasterStateMachine();
-    SnapshotRetentionPolicy snapshotRetentionPolicy =
-        new SnapshotRetentionPolicy() {
-          @Override
-          public int getNumSnapshotsRetained() {
-            return 3;
-          }
-        };
-
-    File storageDir = Utils.createTempDir("./", "snapshot");
-
-    System.out.println(storageDir);
-    final RaftStorage storage =
-        StorageImplUtils.newRaftStorage(storageDir, null, RaftStorage.StartupOption.FORMAT, 100);
-    storage.initialize();
-    SimpleStateMachineStorage simpleStateMachineStorage =
-        (SimpleStateMachineStorage) stateMachine.getStateMachineStorage();
-    simpleStateMachineStorage.init(storage);
-
-    List<Long> indices = new ArrayList<>();
-
-    // Create 5 snapshot files in storage dir.
-    for (int i = 0; i < 5; i++) {
-      final long term = ThreadLocalRandom.current().nextLong(3L, 10L);
-      final long index = ThreadLocalRandom.current().nextLong(100L, 1000L);
-      indices.add(index);
-      File snapshotFile = simpleStateMachineStorage.getSnapshotFile(term, index);
-      snapshotFile.createNewFile();
-      File md5File = new File(snapshotFile.getAbsolutePath() + ".md5");
-      md5File.createNewFile();
-    }
-
-    // following 2 md5 files will be deleted
-    File snapshotFile1 = simpleStateMachineStorage.getSnapshotFile(1, 1);
-    File md5File1 = new File(snapshotFile1.getAbsolutePath() + ".md5");
-    md5File1.createNewFile();
-    File snapshotFile2 = simpleStateMachineStorage.getSnapshotFile(5, 2);
-    File md5File2 = new File(snapshotFile2.getAbsolutePath() + ".md5");
-    md5File2.createNewFile();
-    // this md5 file will not be deleted
-    File snapshotFile3 = simpleStateMachineStorage.getSnapshotFile(11, 1001);
-    File md5File3 = new File(snapshotFile3.getAbsolutePath() + ".md5");
-    md5File3.createNewFile();
-
-    File stateMachineDir = SimpleStateMachineStorageUtil.getSmDir(simpleStateMachineStorage);
-    Assert.assertTrue(stateMachineDir.listFiles().length == 13);
-    simpleStateMachineStorage.cleanupOldSnapshots(snapshotRetentionPolicy);
-    File[] remainingFiles = stateMachineDir.listFiles();
-    Assert.assertTrue(remainingFiles.length == 7);
-
-    Collections.sort(indices);
-    Collections.reverse(indices);
-    List<Long> remainingIndices = indices.subList(0, 3);
-    // check snapshot file and its md5 file management
-    for (File file : remainingFiles) {
-      System.out.println(file.getName());
-      Matcher matcher = SimpleStateMachineStorage.SNAPSHOT_REGEX.matcher(file.getName());
-      if (matcher.matches()) {
-        Assert.assertTrue(remainingIndices.contains(Long.parseLong(matcher.group(2))));
-        Assert.assertTrue(new File(file.getAbsolutePath() + ".md5").exists());
-      }
-    }
-
-    // Attempt to clean up again should not delete any more files.
-    simpleStateMachineStorage.cleanupOldSnapshots(snapshotRetentionPolicy);
-    remainingFiles = stateMachineDir.listFiles();
-    Assert.assertTrue(remainingFiles.length == 7);
-
-    // Test with Retention disabled.
-    // Create 2 snapshot files in storage dir.
-    for (int i = 0; i < 2; i++) {
-      final long term = ThreadLocalRandom.current().nextLong(10L);
-      final long index = ThreadLocalRandom.current().nextLong(1000L);
-      indices.add(index);
-      File snapshotFile = simpleStateMachineStorage.getSnapshotFile(term, index);
-      snapshotFile.createNewFile();
-      File md5File = new File(snapshotFile.getAbsolutePath() + ".md5");
-      md5File.createNewFile();
-    }
-
-    simpleStateMachineStorage.cleanupOldSnapshots(new SnapshotRetentionPolicy() {});
-
-    Assert.assertTrue(stateMachineDir.listFiles().length == 11);
-  }
-
   @Test
   public void testObjSerde() throws IOException, InterruptedException {
     CelebornConf conf = new CelebornConf();
@@ -208,9 +110,9 @@ public class MasterStateMachineSuiteJ extends RatisBaseSuiteJ {
     File tmpFile = File.createTempFile("tef", "test" + System.currentTimeMillis());
 
     Map<String, DiskInfo> disks1 = new HashMap<>();
-    disks1.put("disk1", new DiskInfo("disk1", 64 * 1024 * 1024 * 1024, 100, 100, 0));
-    disks1.put("disk2", new DiskInfo("disk2", 64 * 1024 * 1024 * 1024, 100, 100, 0));
-    disks1.put("disk3", new DiskInfo("disk3", 64 * 1024 * 1024 * 1024, 100, 100, 0));
+    disks1.put("disk1", new DiskInfo("disk1", 64 * 1024 * 1024 * 1024L, 100, 100, 0));
+    disks1.put("disk2", new DiskInfo("disk2", 64 * 1024 * 1024 * 1024L, 100, 100, 0));
+    disks1.put("disk3", new DiskInfo("disk3", 64 * 1024 * 1024 * 1024L, 100, 100, 0));
     Map<UserIdentifier, ResourceConsumption> userResourceConsumption1 =
         JavaUtils.newConcurrentHashMap();
     userResourceConsumption1.put(
@@ -221,9 +123,9 @@ public class MasterStateMachineSuiteJ extends RatisBaseSuiteJ {
         new UserIdentifier("tenant1", "name3"), new ResourceConsumption(3000, 3, 3000, 3));
 
     Map<String, DiskInfo> disks2 = new HashMap<>();
-    disks2.put("disk1", new DiskInfo("disk1", 64 * 1024 * 1024 * 1024, 100, 100, 0));
-    disks2.put("disk2", new DiskInfo("disk2", 64 * 1024 * 1024 * 1024, 100, 100, 0));
-    disks2.put("disk3", new DiskInfo("disk3", 64 * 1024 * 1024 * 1024, 100, 100, 0));
+    disks2.put("disk1", new DiskInfo("disk1", 64 * 1024 * 1024 * 1024L, 100, 100, 0));
+    disks2.put("disk2", new DiskInfo("disk2", 64 * 1024 * 1024 * 1024L, 100, 100, 0));
+    disks2.put("disk3", new DiskInfo("disk3", 64 * 1024 * 1024 * 1024L, 100, 100, 0));
     Map<UserIdentifier, ResourceConsumption> userResourceConsumption2 =
         JavaUtils.newConcurrentHashMap();
     userResourceConsumption2.put(
@@ -234,9 +136,9 @@ public class MasterStateMachineSuiteJ extends RatisBaseSuiteJ {
         new UserIdentifier("tenant2", "name3"), new ResourceConsumption(3000, 3, 3000, 3));
 
     Map<String, DiskInfo> disks3 = new HashMap<>();
-    disks3.put("disk1", new DiskInfo("disk1", 64 * 1024 * 1024 * 1024, 100, 100, 0));
-    disks3.put("disk2", new DiskInfo("disk2", 64 * 1024 * 1024 * 1024, 100, 100, 0));
-    disks3.put("disk3", new DiskInfo("disk3", 64 * 1024 * 1024 * 1024, 100, 100, 0));
+    disks3.put("disk1", new DiskInfo("disk1", 64 * 1024 * 1024 * 1024L, 100, 100, 0));
+    disks3.put("disk2", new DiskInfo("disk2", 64 * 1024 * 1024 * 1024L, 100, 100, 0));
+    disks3.put("disk3", new DiskInfo("disk3", 64 * 1024 * 1024 * 1024L, 100, 100, 0));
     Map<UserIdentifier, ResourceConsumption> userResourceConsumption3 =
         JavaUtils.newConcurrentHashMap();
     userResourceConsumption3.put(
@@ -278,13 +180,19 @@ public class MasterStateMachineSuiteJ extends RatisBaseSuiteJ {
     AppDiskUsageSnapShot originCurrentSnapshot =
         masterStatusSystem.appDiskUsageMetric.currentSnapShot().get();
 
+    masterStatusSystem.workers.add(new WorkerInfo(host1, 9095, 9094, 9093, 9092));
+    masterStatusSystem.workers.add(new WorkerInfo(host2, 9095, 9094, 9093, 9092));
+    masterStatusSystem.workers.add(new WorkerInfo(host3, 9095, 9094, 9093, 9092));
+
     masterStatusSystem.writeMetaInfoToFile(tmpFile);
 
     masterStatusSystem.hostnameSet.clear();
     masterStatusSystem.excludedWorkers.clear();
+    masterStatusSystem.workers.clear();
 
     masterStatusSystem.restoreMetaFromFile(tmpFile);
 
+    Assert.assertEquals(3, masterStatusSystem.workers.size());
     Assert.assertEquals(3, masterStatusSystem.excludedWorkers.size());
     Assert.assertEquals(3, masterStatusSystem.hostnameSet.size());
     Assert.assertEquals(
@@ -295,6 +203,9 @@ public class MasterStateMachineSuiteJ extends RatisBaseSuiteJ {
         masterStatusSystem.appDiskUsageMetric.currentSnapShot().get().topNItems().length);
     Assert.assertEquals(
         originCurrentSnapshot, masterStatusSystem.appDiskUsageMetric.currentSnapShot().get());
-    Assert.assertEquals(originSnapshots, masterStatusSystem.appDiskUsageMetric.snapShots());
+    Assert.assertArrayEquals(originSnapshots, masterStatusSystem.appDiskUsageMetric.snapShots());
+
+    masterStatusSystem.restoreMetaFromFile(tmpFile);
+    Assert.assertEquals(3, masterStatusSystem.workers.size());
   }
 }
