@@ -67,6 +67,8 @@ abstract class HttpService extends Service with Logging {
 
   def isRegistered: String
 
+  def exit(exitType: String): String = throw new UnsupportedOperationException()
+
   def startHttpServer(): Unit = {
     val handlers =
       if (metricsSystem.running) {
@@ -105,19 +107,11 @@ abstract class HttpService extends Service with Logging {
     startHttpServer()
   }
 
-  override def close(): Unit = {
+  override def stop(exitKind: Int): Unit = {
     // may be null when running the unit test
     if (null != httpServer) {
-      httpServer.stop(true)
+      httpServer.stop(exitKind)
     }
-    super.close()
-  }
-
-  override def shutdown(graceful: Boolean): Unit = {
-    // may be null when running the unit test
-    if (null != httpServer) {
-      httpServer.stop(graceful)
-    }
-    super.shutdown(graceful)
+    super.stop(exitKind)
   }
 }
