@@ -20,6 +20,7 @@ package org.apache.celeborn.service.deploy.worker.storage;
 import static org.junit.Assert.assertEquals;
 import static org.junit.Assert.assertNotEquals;
 import static org.junit.Assert.fail;
+import static org.mockito.Mockito.*;
 
 import java.io.File;
 import java.io.IOException;
@@ -82,7 +83,6 @@ public class FileWriterSuiteJ {
   private static final CelebornConf CONF = new CelebornConf();
   public static final Long SPLIT_THRESHOLD = 256 * 1024 * 1024L;
   public static final PartitionSplitMode splitMode = PartitionSplitMode.HARD;
-  public static final PartitionType partitionType = PartitionType.REDUCE;
 
   private static File tempDir = null;
   private static LocalFlusher localFlusher = null;
@@ -138,7 +138,7 @@ public class FileWriterSuiteJ {
 
   public static void setupChunkServer(FileInfo info) {
     FetchHandler handler =
-        new FetchHandler(transConf.getCelebornConf(), transConf) {
+        new FetchHandler(transConf.getCelebornConf(), transConf, mock(WorkerSource.class)) {
           @Override
           public StorageManager storageManager() {
             return new StorageManager(CONF, source);
